# Copyright (c) 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.service.edp import hdfs_helper
from sahara.service.edp import job_utils
from sahara.service.edp.oozie import engine as oozie_engine
from sahara.service.edp.oozie.workflow_creator import workflow_factory
from sahara.service.edp.spark import engine as spark_engine
from sahara.service.edp.storm import engine as storm_engine
from sahara.utils import edp

JOB_TYPE_HIVE = edp.JOB_TYPE_HIVE
JOB_TYPE_SPARK = edp.JOB_TYPE_SPARK
JOB_TYPE_JAVA = edp.JOB_TYPE_JAVA
JOB_TYPE_SHELL = edp.JOB_TYPE_SHELL
JOB_TYPE_PIG = edp.JOB_TYPE_PIG
JOB_TYPE_STORM = edp.JOB_TYPE_STORM
JOB_TYPE_PYLEUS = edp.JOB_TYPE_PYLEUS
JOB_TYPE_MAPREDUCE = edp.JOB_TYPE_MAPREDUCE
JOB_TYPE_MAPREDUCE_STREAMING = edp.JOB_TYPE_MAPREDUCE_STREAMING
JOB_TYPES_ALL = edp.JOB_TYPES_ALL
JOB_STATUS_SUCCEEDED = edp.JOB_STATUS_SUCCEEDED


class PluginsStormJobEngine(storm_engine.StormJobEngine):

    def __init__(self, cluster, **kwargs):
        super(PluginsStormJobEngine, self).__init__(cluster)


class PluginsStormPyleusJobEngine(storm_engine.StormPyleusJobEngine):

    def __init__(self, cluster, **kwargs):
        super(PluginsStormPyleusJobEngine, self).__init__(cluster)


class PluginsSparkJobEngine(spark_engine.SparkJobEngine):

    def __init__(self, cluster, **kwargs):
        super(PluginsSparkJobEngine, self).__init__(cluster)


class PluginsSparkShellJobEngine(spark_engine.SparkShellJobEngine):

    def __init__(self, cluster, **kwargs):
        super(PluginsSparkShellJobEngine, self).__init__(cluster)


class PluginsOozieJobEngine(oozie_engine.OozieJobEngine):

    def __init__(self, cluster, **kwargs):
        super(PluginsOozieJobEngine, self).__init__(cluster)


def get_hive_shared_conf_path(hdfs_user, **kwargs):
    return edp.get_hive_shared_conf_path(hdfs_user)


def compare_job_type(job_type, *args, **kwargs):
    return edp.compare_job_type(job_type, *args, **kwargs)


def get_builtin_binaries(job, configs, **kwargs):
    return edp.get_builtin_binaries(job, configs)


def create_dir_hadoop2(r, dir_name, hdfs_user, **kwargs):
    hdfs_helper.create_dir_hadoop2(r, dir_name, hdfs_user)


def create_hbase_common_lib(r, **kwargs):
    hdfs_helper.create_hbase_common_lib(r)


def get_plugin(cluster, **kwargs):
    return job_utils.get_plugin(cluster)


def get_possible_job_config(job_type, **kwargs):
    return workflow_factory.get_possible_job_config(job_type)


def get_possible_mapreduce_configs(**kwargs):
    return workflow_factory.get_possible_mapreduce_configs()



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools

import six

from sahara.conductor import resource
from sahara import exceptions as exc
from sahara.utils.openstack import glance


PROP_DESCR = '_sahara_description'
PROP_USERNAME = '_sahara_username'
PROP_TAG = '_sahara_tag_'
PROP_ALL_TAGS = '_all_tags'


def image_manager():
    return SaharaImageManager()


def wrap_entity(func):
    @functools.wraps(func)
    def handle(*args, **kwargs):
        res = func(*args, **kwargs)
        if isinstance(res, list):
            images = []
            for image in res:
                image = _transform_image_props(image)
                images.append(resource.ImageResource(image))
            return images
        else:
            res = _transform_image_props(res)
            return resource.ImageResource(res)
    return handle


def _get_all_tags(image_props):
    tags = []
    for key, value in image_props.iteritems():
        if key.startswith(PROP_TAG) and value:
            tags.append(key)
    return tags


def _get_meta_prop(image_props, prop, default=None):
    if PROP_ALL_TAGS == prop:
        return _get_all_tags(image_props)
    return image_props.get(prop, default)


def _parse_tags(image_props):
    tags = _get_meta_prop(image_props, PROP_ALL_TAGS)
    return [t.replace(PROP_TAG, "") for t in tags]


def _serialize_metadata(image):
    data = {}
    for key, value in image.iteritems():
        if key.startswith('_sahara') and value:
            data[key] = value
    return data


def _get_compat_values(image):
    data = {}
    # TODO(vgridnev): Drop these values from APIv2
    data["OS-EXT-IMG-SIZE:size"] = image.size
    data['metadata'] = _serialize_metadata(image)
    data["minDisk"] = getattr(image, 'min_disk', 0)
    data["minRam"] = getattr(image, 'min_ram', 0)
    data["progress"] = getattr(image, 'progress', 100)
    data["status"] = image.status.upper()
    data['created'] = image.created_at
    data['updated'] = image.updated_at
    return data


def _transform_image_props(image):
    data = _get_compat_values(image)
    data['username'] = _get_meta_prop(image, PROP_USERNAME, "")
    data['description'] = _get_meta_prop(image, PROP_DESCR, "")
    data['tags'] = _parse_tags(image)
    data['id'] = image.id
    data["name"] = image.name
    return data


def _ensure_tags(tags):
    if not tags:
        return []
    return [tags] if isinstance(tags, six.string_types) else tags


class SaharaImageManager(object):
    """SaharaImageManager

    This class is intermediate layer between sahara and glanceclient.v2.images.
    It provides additional sahara properties for image such as description,
    image tags and image username.
    """
    def __init__(self):
        self.client = glance.client().images

    @wrap_entity
    def get(self, image_id):
        image = self.client.get(image_id)
        return image

    @wrap_entity
    def find(self, **kwargs):
        images = self.client.list(**kwargs)
        num_matches = len(images)
        if num_matches == 0:
            raise exc.NotFoundException(kwargs, "No images matching %s.")
        elif num_matches > 1:
            raise exc.NoUniqueMatchException(response=images, query=kwargs)
        else:
            return images[0]

    @wrap_entity
    def list(self):
        return list(self.client.list())

    def set_meta(self, image_id, meta):
        self.client.update(image_id, remove_props=None, **meta)

    def delete_meta(self, image_id, meta_list):
        self.client.update(image_id, remove_props=meta_list)

    def set_image_info(self, image_id, username, description=None):
        """Sets human-readable information for image.

        For example:
            Ubuntu 15 x64 with Java 1.7 and Apache Hadoop 2.1, ubuntu
        """
        meta = {PROP_USERNAME: username}
        if description:
            meta[PROP_DESCR] = description
        self.set_meta(image_id, meta)

    def unset_image_info(self, image_id):
        """Unsets all Sahara-related information.

        It removes username, description and tags from the specified image.
        """
        image = self.get(image_id)
        meta = [PROP_TAG + tag for tag in image.tags]
        if image.description is not None:
            meta += [PROP_DESCR]
        if image.username is not None:
            meta += [PROP_USERNAME]
        self.delete_meta(image_id, meta)

    def tag(self, image_id, tags):
        """Adds tags to the specified image."""
        tags = _ensure_tags(tags)
        self.set_meta(image_id, {PROP_TAG + tag: 'True' for tag in tags})

    def untag(self, image_id, tags):
        """Removes tags from the specified image."""
        tags = _ensure_tags(tags)
        self.delete_meta(image_id, [PROP_TAG + tag for tag in tags])

    def list_by_tags(self, tags):
        """Returns images having all of the specified tags."""
        tags = _ensure_tags(tags)
        return [i for i in self.list() if set(tags).issubset(i.tags)]

    def list_registered(self, name=None, tags=None):
        tags = _ensure_tags(tags)
        images_list = [i for i in self.list()
                       if i.username and set(tags).issubset(i.tags)]
        if name:
            return [i for i in images_list if i.name == name]
        else:
            return images_list

    def get_registered_image(self, image_id):
        img = self.get(image_id)
        if img.username:
            return img
        else:
            raise exc.ImageNotRegistered(image_id)



# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

import sahara.exceptions as ex
from sahara.service.castellan import config as castellan
from sahara.service.edp.binary_retrievers import internal_swift as i_s
from sahara.tests.unit import base


class TestInternalSwift(base.SaharaTestCase):
    def setUp(self):
        super(TestInternalSwift, self).setUp()
        castellan.validate_config()

    def test__get_raw_data(self):
        client_instance = mock.Mock()
        client_instance.head_object = mock.Mock()
        client_instance.get_object = mock.Mock()

        job_binary = mock.Mock()
        job_binary.url = 'swift://container/object'

        # an object that is too large should raise an exception
        header = {'content-length': '2048'}
        client_instance.head_object.return_value = header
        self.override_config('job_binary_max_KB', 1)
        self.assertRaises(ex.DataTooBigException,
                          i_s._get_raw_data,
                          job_binary,
                          client_instance)
        client_instance.head_object.assert_called_once_with('container',
                                                            'object')

        # valid return
        header = {'content-length': '4'}
        body = 'data'
        client_instance.head_object.return_value = header
        client_instance.get_object.return_value = (header, body)
        self.assertEqual(body, i_s._get_raw_data(job_binary, client_instance))
        client_instance.get_object.assert_called_once_with('container',
                                                           'object')

    def test__validate_job_binary_url(self):
        @i_s._validate_job_binary_url
        def empty_method(job_binary):
            pass

        job_binary = mock.Mock()

        # bad swift url should raise an exception
        job_binary.url = 'notswift://container/object'
        self.assertRaises(ex.BadJobBinaryException,
                          empty_method,
                          job_binary)

        # specifying a container should raise an exception
        job_binary.url = 'swift://container'
        self.assertRaises(ex.BadJobBinaryException,
                          empty_method,
                          job_binary)

    @mock.patch(
        'sahara.service.edp.binary_retrievers.internal_swift._get_raw_data')
    @mock.patch('sahara.utils.openstack.swift.client')
    def test_get_raw_data(self, swift_client, _get_raw_data):
        client_instance = mock.Mock()
        swift_client.return_value = client_instance

        job_binary = mock.Mock()
        job_binary.url = 'swift://container/object'

        # embedded credentials
        job_binary.extra = dict(user='test', password='secret')
        i_s.get_raw_data(job_binary)
        swift_client.assert_called_with(username='test',
                                        password='secret')
        _get_raw_data.assert_called_with(job_binary, client_instance)

        # proxy configs should override embedded credentials
        proxy_configs = dict(proxy_username='proxytest',
                             proxy_password='proxysecret',
                             proxy_trust_id='proxytrust')
        i_s.get_raw_data(job_binary, proxy_configs)
        swift_client.assert_called_with(username='proxytest',
                                        password='proxysecret',
                                        trust_id='proxytrust')
        _get_raw_data.assert_called_with(job_binary, client_instance)

    @mock.patch('sahara.utils.openstack.base.url_for')
    @mock.patch('sahara.context.ctx')
    @mock.patch(
        'sahara.service.edp.binary_retrievers.internal_swift._get_raw_data')
    @mock.patch('swiftclient.Connection')
    def test_get_raw_data_with_context(self, swift_client, _get_raw_data, ctx,
                                       url_for):
        client_instance = mock.Mock()
        swift_client.return_value = client_instance
        test_context = mock.Mock()
        test_context.auth_token = 'testtoken'
        test_context.auth_plugin = None
        ctx.return_value = test_context
        url_for.return_value = 'url_for'
        job_binary = mock.Mock()
        job_binary.url = 'swift://container/object'

        job_binary.extra = dict(user='test', password='secret')
        i_s.get_raw_data_with_context(job_binary)
        self.assertEqual([mock.call(
            auth_version='3',
            cacert=None, insecure=False,
            max_backoff=10,
            preauthtoken='testtoken',
            preauthurl='url_for', retries=5,
            retry_on_ratelimit=True, starting_backoff=10)],
            swift_client.call_args_list)
        _get_raw_data.assert_called_with(job_binary, client_instance)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from oslo_log import log as logging

from sahara import conductor as cond
from sahara import context
from sahara.plugins import provisioning as common_configs
from sahara.utils import cluster as c_u


CONF = cfg.CONF

LOG = logging.getLogger(__name__)
conductor = cond.API

ntp_opts = [
    cfg.StrOpt('default_ntp_server',
               default="pool.ntp.org",
               help="Default ntp server for time sync")
]

CONF.register_opts(ntp_opts)


def _sudo(remote, cmd):
    remote.execute_command(cmd, run_as_root=True)


def _restart_ntp(remote):
    distrib = remote.get_os_distrib()
    cmd = "service %s restart"
    if distrib == 'ubuntu':
        cmd = cmd % "ntp"
    else:
        cmd = cmd % "ntpd"

    _sudo(remote, cmd)


def _verify_installation(remote):
    distrib = remote.get_os_distrib()
    if distrib == 'ubuntu':
        return remote.execute_command("dpkg -s ntp")
    else:
        return remote.execute_command("rpm -q ntp")


def _check_ntp_installed(remote):
    try:
        exit_code, stdout = _verify_installation(remote)
        if exit_code != 0:
            return False
        return True
    except Exception:
        return False


def _configure_ntp_on_instance(instance, url):
    with context.set_current_instance_id(instance.instance_id):
        LOG.debug("Configuring ntp server")
        with instance.remote() as r:
            if not _check_ntp_installed(r):
                # missing ntp service
                LOG.warning("Unable to configure NTP service")
                return

            r.prepend_to_file(
                "/etc/ntp.conf", "server {url} iburst\n".format(url=url),
                run_as_root=True)
            _restart_ntp(r)
            try:
                _sudo(r, "ntpdate -u {url}".format(url=url))
            except Exception as e:
                LOG.debug("Update time on VM failed with error: %s", e)
            LOG.info("NTP successfully configured")


def is_ntp_enabled(cluster):
    target = common_configs.NTP_ENABLED.applicable_target
    name = common_configs.NTP_ENABLED.name
    cl_configs = cluster.cluster_configs
    if target not in cl_configs or name not in cl_configs[target]:
        return common_configs.NTP_ENABLED.default_value
    return cl_configs[target][name]


def retrieve_ntp_server_url(cluster):
    target = common_configs.NTP_URL.applicable_target
    name = common_configs.NTP_URL.name
    cl_configs = cluster.cluster_configs
    if target not in cl_configs or name not in cl_configs[target]:
        return CONF.default_ntp_server
    return cl_configs[target][name]


def configure_ntp(cluster_id, instance_ids=None):
    cluster = conductor.cluster_get(context.ctx(), cluster_id)
    if not is_ntp_enabled(cluster):
        LOG.debug("Don't configure NTP on cluster")
        return
    instances = c_u.get_instances(cluster, instance_ids)
    url = retrieve_ntp_server_url(cluster)
    with context.ThreadGroup() as tg:
        for instance in instances:
            tg.spawn("configure-ntp-%s" % instance.instance_name,
                     _configure_ntp_on_instance, instance, url)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

from sahara import main
from sahara.plugins import base as pb


class BasePluginsSupportTest(testtools.TestCase):

    def setUp(self):
        super(BasePluginsSupportTest, self).setUp()
        main.CONF.set_override('plugins', ['fake', 'cdh', 'spark'])
        pb.setup_plugins()

    def test_plugins_loaded(self):
        plugins = [p.name for p in pb.PLUGINS.get_plugins()]
        self.assertIn('fake', plugins)



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc

import six

from sahara import exceptions as ex
from sahara.plugins import base as plugins_base


@six.add_metaclass(abc.ABCMeta)
class JobBinaryType(object):
    @plugins_base.required_with_default
    def prepare_cluster(self, job_binary, **kwargs):
        """Makes a cluster ready to use this job binary

        Different implementations for each job binary type for Manila it will
        be mount the share, for Swift verify credentials, and so on

        :param job_binary: The object handle to a job binary
        :returns: None
        """
        pass

    @plugins_base.required
    def copy_binary_to_cluster(self, job_binary, **kwargs):
        """Get the path for the binary in a cluster

        If necessary, pull binary data from the binary store, and copy that
        data to a useful path on the cluster. Then returns a valid
        FS path for the job binary in the cluster

        :param job_binary: The object handle to a job binary

        :returns: String representing the local path
        """

        # TODO(mariannelm): currently for the job binaries it's true
        # that the raw data must be available at a FS path in the cluster, but
        # for most of the job binary types there's no need to keep this data
        # in the cluster after the job is done, so it would be a good thing to
        # have a method responsible for removing the job binary raw data
        # after the end of the job
        return None

    @plugins_base.required_with_default
    def validate(self, data, **kwargs):
        """Method that validate the data passed through the API

        This method will be executed during the job binary creation and
        update

        :raise: If data is invalid, InvalidDataException
        """
        pass

    @plugins_base.optional
    def _validate_url(self, url):
        """Auxiliary method used by the validate method"""
        pass

    @plugins_base.required_with_default
    def validate_job_location_format(self, entry):
        """Checks whether or not the API entry is valid

        :param entry: String that represents a job binary url

        :returns: True if this entry is valid, False otherwhise

        """
        return True

    @plugins_base.required_with_default
    def get_raw_data(self, job_binary, **kwargs):
        """Get the raw binary

        Used only by the API, if the type doesn't support this operation it
        should raise NotImplementedException

        :param job_binary: The object handle to a job binary

        :returns: Raw binary

        """
        raise ex.NotImplementedException()

    @plugins_base.optional
    def _generate_valid_path(self, job_binary):
        """Generates a valid FS path for the binary be placed"""
        return '/tmp/' + job_binary.name



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import mock
from oslo_utils import uuidutils
import testtools

from sahara import conductor as cond
from sahara.service.edp.data_sources import manager as ds_manager
from sahara.service.edp import job_utils
from sahara.tests.unit.service.edp import edp_test_utils as u

conductor = cond.API


class JobUtilsTestCase(testtools.TestCase):

    def setUp(self):
        super(JobUtilsTestCase, self).setUp()
        ds_manager.setup_data_sources()

    def test_args_may_contain_data_sources(self):
        job_configs = None

        # No configs, default false
        by_name, by_uuid = job_utils.may_contain_data_source_refs(job_configs)
        self.assertFalse(by_name | by_uuid)

        # Empty configs, default false
        job_configs = {'configs': {}}
        by_name, by_uuid = job_utils.may_contain_data_source_refs(job_configs)
        self.assertFalse(by_name | by_uuid)

        job_configs['configs'] = {job_utils.DATA_SOURCE_SUBST_NAME: True,
                                  job_utils.DATA_SOURCE_SUBST_UUID: True}
        by_name, by_uuid = job_utils.may_contain_data_source_refs(job_configs)
        self.assertTrue(by_name & by_uuid)

        job_configs['configs'][job_utils.DATA_SOURCE_SUBST_NAME] = False
        by_name, by_uuid = job_utils.may_contain_data_source_refs(job_configs)
        self.assertFalse(by_name)
        self.assertTrue(by_uuid)

        job_configs['configs'][job_utils.DATA_SOURCE_SUBST_UUID] = False
        by_name, by_uuid = job_utils.may_contain_data_source_refs(job_configs)
        self.assertFalse(by_name | by_uuid)

        job_configs['configs'] = {job_utils.DATA_SOURCE_SUBST_NAME: 'True',
                                  job_utils.DATA_SOURCE_SUBST_UUID: 'Fish'}
        by_name, by_uuid = job_utils.may_contain_data_source_refs(job_configs)
        self.assertTrue(by_name)
        self.assertFalse(by_uuid)

    def test_find_possible_data_source_refs_by_name(self):
        id = uuidutils.generate_uuid()
        job_configs = {}
        self.assertEqual([],
                         job_utils.find_possible_data_source_refs_by_name(
                             job_configs))

        name_ref = job_utils.DATA_SOURCE_PREFIX+'name'
        name_ref2 = name_ref+'2'

        job_configs = {'args': ['first', id],
                       'configs': {'config': 'value'},
                       'params': {'param': 'value'}}
        self.assertEqual([],
                         job_utils.find_possible_data_source_refs_by_name(
                             job_configs))

        job_configs = {'args': [name_ref, id],
                       'configs': {'config': 'value'},
                       'params': {'param': 'value'}}
        self.assertEqual(
            ['name'],
            job_utils.find_possible_data_source_refs_by_name(job_configs))

        job_configs = {'args': ['first', id],
                       'configs': {'config': name_ref},
                       'params': {'param': 'value'}}
        self.assertEqual(
            ['name'],
            job_utils.find_possible_data_source_refs_by_name(job_configs))

        job_configs = {'args': ['first', id],
                       'configs': {'config': 'value'},
                       'params': {'param': name_ref}}
        self.assertEqual(
            ['name'],
            job_utils.find_possible_data_source_refs_by_name(job_configs))

        job_configs = {'args': [name_ref, name_ref2, id],
                       'configs': {'config': name_ref},
                       'params': {'param': name_ref}}
        self.assertItemsEqual(
            ['name', 'name2'],
            job_utils.find_possible_data_source_refs_by_name(job_configs))

    def test_find_possible_data_source_refs_by_uuid(self):
        job_configs = {}

        name_ref = job_utils.DATA_SOURCE_PREFIX+'name'

        self.assertEqual([],
                         job_utils.find_possible_data_source_refs_by_uuid(
                             job_configs))

        id = uuidutils.generate_uuid()
        job_configs = {'args': ['first', name_ref],
                       'configs': {'config': 'value'},
                       'params': {'param': 'value'}}
        self.assertEqual([],
                         job_utils.find_possible_data_source_refs_by_uuid(
                             job_configs))

        job_configs = {'args': [id, name_ref],
                       'configs': {'config': 'value'},
                       'params': {'param': 'value'}}
        self.assertEqual(
            [id],
            job_utils.find_possible_data_source_refs_by_uuid(job_configs))

        job_configs = {'args': ['first', name_ref],
                       'configs': {'config': id},
                       'params': {'param': 'value'}}
        self.assertEqual(
            [id],
            job_utils.find_possible_data_source_refs_by_uuid(job_configs))

        job_configs = {'args': ['first', name_ref],
                       'configs': {'config': 'value'},
                       'params': {'param': id}}
        self.assertEqual(
            [id],
            job_utils.find_possible_data_source_refs_by_uuid(job_configs))

        id2 = uuidutils.generate_uuid()
        job_configs = {'args': [id, id2, name_ref],
                       'configs': {'config': id},
                       'params': {'param': id}}
        self.assertItemsEqual([id, id2],
                              job_utils.find_possible_data_source_refs_by_uuid(
                                  job_configs))

    @mock.patch('sahara.context.ctx')
    @mock.patch('sahara.conductor.API.data_source_get_all')
    def test_resolve_data_source_refs(self, data_source_get_all, ctx):

        ctx.return_value = 'dummy'

        name_ref = job_utils.DATA_SOURCE_PREFIX+'input'
        job_exec_id = uuidutils.generate_uuid()

        input_url = "swift://container/input"
        input = u.create_data_source(input_url,
                                     name="input",
                                     id=uuidutils.generate_uuid())

        output = u.create_data_source("swift://container/output.%JOB_EXEC_ID%",
                                      name="output",
                                      id=uuidutils.generate_uuid())
        output_url = "swift://container/output." + job_exec_id

        by_name = {'input': input,
                   'output': output}

        by_id = {input.id: input,
                 output.id: output}

        # Pretend to be the database
        def _get_all(ctx, **kwargs):
            name = kwargs.get('name')
            if name in by_name:
                name_list = [by_name[name]]
            else:
                name_list = []

            id = kwargs.get('id')
            if id in by_id:
                id_list = [by_id[id]]
            else:
                id_list = []
            return list(set(name_list + id_list))

        data_source_get_all.side_effect = _get_all

        job_configs = {
            'configs': {
                job_utils.DATA_SOURCE_SUBST_NAME: True,
                job_utils.DATA_SOURCE_SUBST_UUID: True},
            'args': [name_ref, output.id, input.id]}
        urls = {}
        ds, nc = job_utils.resolve_data_source_references(job_configs,
                                                          job_exec_id, urls)
        self.assertEqual(2, len(ds))
        self.assertEqual([input.url, output_url, input.url], nc['args'])

        # Substitution not enabled
        job_configs['configs'] = {job_utils.DATA_SOURCE_SUBST_NAME: False,
                                  job_utils.DATA_SOURCE_SUBST_UUID: False}
        ds, nc = job_utils.resolve_data_source_references(job_configs,
                                                          job_exec_id, {})
        self.assertEqual(0, len(ds))
        self.assertEqual(job_configs['args'], nc['args'])
        self.assertEqual(job_configs['configs'], nc['configs'])

        # Substitution enabled but no values to modify
        job_configs['configs'] = {job_utils.DATA_SOURCE_SUBST_NAME: True,
                                  job_utils.DATA_SOURCE_SUBST_UUID: True}
        job_configs['args'] = ['val1', 'val2', 'val3']
        ds, nc = job_utils.resolve_data_source_references(job_configs,
                                                          job_exec_id, {})
        self.assertEqual(0, len(ds))
        self.assertEqual(nc['args'], job_configs['args'])
        self.assertEqual(nc['configs'], job_configs['configs'])

    def test_to_url_dict(self):
        data_source_urls = {'1': ('1_native', '1_runtime'),
                            '2': ('2_native', '2_runtime')}
        self.assertItemsEqual({'1': '1_native',
                               '2': '2_native'},
                              job_utils.to_url_dict(data_source_urls))

        self.assertItemsEqual({'1': '1_runtime',
                               '2': '2_runtime'},
                              job_utils.to_url_dict(data_source_urls,
                                                    runtime=True))

    @mock.patch('sahara.service.edp.hdfs_helper.configure_cluster_for_hdfs')
    def test_prepare_cluster_for_ds(self, configure):
        data_source_urls = {'1': '1_runtime',
                            '2': '2_runtime'}

        data_source = mock.Mock()
        data_source.type = 'hdfs'
        data_source.id = '1'

        cluster = mock.Mock()
        job_configs = mock.Mock()

        job_utils.prepare_cluster_for_ds([data_source], cluster, job_configs,
                                         data_source_urls)

        configure.assert_called_once()
        configure.assert_called_with(cluster, '1_runtime')



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

HEALTH_STATUS_GREEN = "GREEN"
HEALTH_STATUS_YELLOW = "YELLOW"
HEALTH_STATUS_RED = "RED"
HEALTH_STATUS_CHECKING = "CHECKING"
HEALTH_STATUS_DONE = [
    HEALTH_STATUS_GREEN,
    HEALTH_STATUS_YELLOW,
    HEALTH_STATUS_RED,
]

VERIFICATIONS_START_OPS = "START"

VERIFICATIONS_OPS = [
    VERIFICATIONS_START_OPS,
]

CONF = cfg.CONF

health_opts = [
    cfg.BoolOpt('verification_enable', default=True,
                help="Option to enable verifications for all clusters"),
    cfg.IntOpt('verification_periodic_interval', default=600,
               help="Interval between two consecutive periodic tasks for "
                    "verifications, in seconds."),
    cfg.IntOpt('verification_timeout', default=600,
               help="Time limit for health check function, in seconds.")
]
health_opts_group = cfg.OptGroup(
    'cluster_verifications', title='Options to configure verifications')
CONF.register_group(group=health_opts_group)
CONF.register_opts(health_opts, group=health_opts_group)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sahara.exceptions as e
from sahara.i18n import _


class NodeGroupCannotBeScaled(e.SaharaException):
    def __init__(self, ng_name, reason):
        self.message = _("Chosen node group %(ng_name)s cannot be scaled : "
                         "%(reason)s") % {"ng_name": ng_name, "reason": reason}
        self.code = "NODE_GROUP_CANNOT_BE_SCALED"
        super(NodeGroupCannotBeScaled, self).__init__()


class DecommissionError(e.SaharaException):
    code = "DECOMMISSION_ERROR"
    message = _("Failed to decommission cluster")

    def __init__(self, message=None):
        if message:
            self.message = message
        super(DecommissionError, self).__init__()


class ClusterCannotBeScaled(e.SaharaException):
    def __init__(self, cluster_name, reason):
        self.message = _("Cluster %(cluster_name)s cannot be scaled : "
                         "%(reason)s") % {"cluster_name": cluster_name,
                                          "reason": reason}
        self.code = "CLUSTER_CANNOT_BE_SCALED"
        super(ClusterCannotBeScaled, self).__init__()


class RequiredServiceMissingException(e.SaharaException):
    """Exception indicating that a required service has not been deployed."""

    def __init__(self, service_name, required_by=None):
        self.message = (_('Cluster is missing a service: %s')
                        % service_name)
        if required_by:
            self.message = (_('%(message)s, required by service: '
                              '%(required_by)s')
                            % {'message': self.message,
                               'required_by': required_by})

        self.code = 'MISSING_SERVICE'

        super(RequiredServiceMissingException, self).__init__()


class InvalidComponentCountException(e.SaharaException):
    """Exception indicating invalid number of components in a cluster.

    A message indicating a number of components cluster should contain and
    an invalid number of components are being deployed in a cluster.
    """

    def __init__(self, component, expected_count, count, description=None):
        message = _("Hadoop cluster should contain %(expected_count)s "
                    "%(component)s component(s)."
                    " Actual %(component)s count is %(count)s")

        if description:
            message = ("%(message)s. %(description)s"
                       % {'message': message, 'description': description})

        self.message = message % {"expected_count": expected_count,
                                  "component": component, "count": count}

        self.code = "INVALID_COMPONENT_COUNT"

        super(InvalidComponentCountException, self).__init__()


class InvalidClusterTopology(e.SaharaException):
    """Exception indicating another problems in a cluster topology,

    which is different from InvalidComponentCountException and
    RequiredServiceMissingException.
    """
    code = "INVALID_TOPOLOGY"
    message = _("Cluster has invalid topology: {description}")

    def __init__(self, description):
        self.message = self.message.format(description=description)
        super(InvalidClusterTopology, self).__init__()


class HadoopProvisionError(e.SaharaException):
    """Exception indicating that cluster provisioning failed.

    A message indicating the reason for failure must be provided.
    """

    base_message = _("Failed to Provision Hadoop Cluster: %s")

    def __init__(self, message):
        self.code = "HADOOP_PROVISION_FAILED"
        self.message = self.base_message % message

        super(HadoopProvisionError, self).__init__()


class NameNodeHAConfigurationError(e.SaharaException):
    """Exception indicating that hdp or cdh HDFS HA failed.

    A message indicating the reason for failure must be provided.
    """

    base_message = _("NameNode High Availability: %s")

    def __init__(self, message):
        self.code = "NAMENODE_HIGHAVAILABILITY_CONFIGURATION_FAILED"
        self.message = self.base_message % message

        super(NameNodeHAConfigurationError, self).__init__()


class ResourceManagerHAConfigurationError(e.SaharaException):
    """Exception indicating that cdh YARN HA failed.

    A message indicating the reason for failure must be provided.
    """

    base_message = _("ResourceManager High Availability: %s")

    def __init__(self, message):
        self.code = "RESOURCEMANAGER_HIGHAVAILABILITY_CONFIGURATION_FAILED"
        self.message = self.base_message % message

        super(ResourceManagerHAConfigurationError, self).__init__()


class ImageValidationSpecificationError(e.SaharaException):
    """Exception indicating that an image validation spec is in error."""

    base_message = _("Image validation spec is in error: %s")

    def __init__(self, message, *args):
        self.code = "IMAGE_SPECIFICATION_ERROR"
        self.message = self.base_message % message

        super(ImageValidationSpecificationError, self).__init__()


class ImageValidationError(e.SaharaException):
    """Exception indicating that an image has failed validation."""

    base_message = _("Image has failed validation: %s")

    def __init__(self, message):
        self.code = "IMAGE_VALIDATION_FAILED"
        self.message = self.base_message % message

        super(ImageValidationError, self).__init__()


class AllValidationsFailedError(ImageValidationError):
    """Exception indicating that all validations in an any block failed."""

    sub_message = _("All validations have failed: %s")

    def __init__(self, exceptions):
        data = ";".join(ex.message for ex in exceptions)
        message = self.sub_message % data

        super(AllValidationsFailedError, self).__init__(message)


class InvalidVolumeSizeException(e.SaharaException):
    """Exception indicating invalid configuration of components in a cluster.

    """

    def __init__(self, volume_size, reserved):
        message = _("Volume size: %(volume_size)s GB should be greater than "
                    "value of \"dfs_datanode_du_reserved\": %(reserved)s GB")
        self.message = message % {"volume_size": volume_size,
                                  "reserved": reserved}
        self.code = "INVALID_CONFIGURATION"

        super(InvalidVolumeSizeException, self).__init__()


class PluginInvalidDataException(e.InvalidDataException):

    code = "INVALID_DATA"
    message = _("Data is invalid")


class PluginConfigurationError(e.ConfigurationError):
    code = "CONFIGURATION_ERROR"
    message = _("The configuration has failed")


class PluginNotFoundException(e.NotFoundException):
    code = "NOT_FOUND"
    message_template = _("Object '%s' is not found")


class SaharaPluginException(e.SaharaException):
    code = "Sahara Plugin Exception"
    message = ""


class PluginRemoteCommandException(e.RemoteCommandException):
    code = "REMOTE_COMMAND_FAILED"
    message_template = _("Error during command execution: \"%s\"")



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1 import session as keystone
from oslo_config import cfg
from oslo_log import log as logging

from sahara import exceptions as ex
from sahara.i18n import _


CONF = cfg.CONF
LOG = logging.getLogger(__name__)

_SESSION_CACHE = None

SESSION_TYPE_CINDER = 'cinder'
SESSION_TYPE_KEYSTONE = 'keystone'
SESSION_TYPE_NEUTRON = 'neutron'
SESSION_TYPE_NOVA = 'nova'
SESSION_TYPE_GLANCE = 'glance'
SESSION_TYPE_INSECURE = 'insecure'
SESSION_TYPE_HEAT = 'heat'


def cache():
    global _SESSION_CACHE
    if not _SESSION_CACHE:
        _SESSION_CACHE = SessionCache()
    return _SESSION_CACHE


class SessionCache(object):
    '''A cache of keystone Session objects

    When a requested Session is not currently cached, it will be
    acquired from specific information in this module. Sessions should
    be referenced by their OpenStack project name and not the service
    name, this is to allow for multiple service implementations
    while retaining the ability to generate Session objects. In all
    cases, the constant values in this module should be used to
    communicate the session type.

    '''
    def __init__(self):
        '''create a new SessionCache'''
        self._sessions = {}
        self._session_funcs = {
            SESSION_TYPE_CINDER: self.get_cinder_session,
            SESSION_TYPE_KEYSTONE: self.get_keystone_session,
            SESSION_TYPE_NEUTRON: self.get_neutron_session,
            SESSION_TYPE_NOVA: self.get_nova_session,
            SESSION_TYPE_GLANCE: self.get_glance_session,
            SESSION_TYPE_INSECURE: self.get_insecure_session,
            SESSION_TYPE_HEAT: self.get_heat_session,
        }

    def _set_session(self, session_type, session):
        '''Set the session for a given type.

        :param session_type: the type of session to set.

        :param session: the session to associate with the type
        '''
        self._sessions[session_type] = session

    def get_session(self, session_type=SESSION_TYPE_INSECURE):
        '''Return a Session for the requested type

        :param session_type: the type of Session to get, if None an insecure
        session will be returned.

        :raises SaharaException: if the requested session type is not
        found.
        '''
        session_function = self._session_funcs.get(session_type)
        if session_function:
            return session_function()
        else:
            LOG.error('Requesting an unknown session type (type: {type})'.
                      format(type=session_type))
            raise ex.SaharaException(
                _('Session type {type} not recognized').
                format(type=session_type))

    def get_insecure_session(self):
        session = self._sessions.get(SESSION_TYPE_INSECURE)
        if not session:
            session = keystone.Session(verify=False)
            self._set_session(SESSION_TYPE_INSECURE, session)
        return session

    def get_cinder_session(self):
        session = self._sessions.get(SESSION_TYPE_CINDER)
        if not session:
            if not CONF.cinder.api_insecure:
                session = keystone.Session(
                    verify=CONF.cinder.ca_file or True)
            else:
                session = self.get_insecure_session()
            self._set_session(SESSION_TYPE_CINDER, session)
        return session

    def get_keystone_session(self):
        session = self._sessions.get(SESSION_TYPE_KEYSTONE)
        if not session:
            if not CONF.keystone.api_insecure:
                session = keystone.Session(
                    verify=CONF.keystone.ca_file or True)
            else:
                session = self.get_insecure_session()
            self._set_session(SESSION_TYPE_KEYSTONE, session)
        return session

    def get_neutron_session(self):
        session = self._sessions.get(SESSION_TYPE_NEUTRON)
        if not session:
            if not CONF.neutron.api_insecure:
                session = keystone.Session(
                    verify=CONF.neutron.ca_file or True)
            else:
                session = self.get_insecure_session()
            self._set_session(SESSION_TYPE_NEUTRON, session)
        return session

    def get_nova_session(self):
        session = self._sessions.get(SESSION_TYPE_NOVA)
        if not session:
            if not CONF.nova.api_insecure:
                session = keystone.Session(
                    verify=CONF.nova.ca_file or True)
            else:
                session = self.get_insecure_session()
            self._set_session(SESSION_TYPE_NOVA, session)
        return session

    def get_glance_session(self):
        session = self._sessions.get(SESSION_TYPE_GLANCE)
        if not session:
            if not CONF.glance.api_insecure:
                session = keystone.Session(verify=CONF.glance.ca_file or True)
            else:
                session = self.get_insecure_session()
            self._set_session(SESSION_TYPE_GLANCE, session)
        return session

    def get_heat_session(self):
        session = self._sessions.get(SESSION_TYPE_HEAT)
        if not session:
            if not CONF.heat.api_insecure:
                session = keystone.Session(verify=CONF.heat.ca_file or True)
            else:
                session = self.get_insecure_session()
            self._set_session(SESSION_TYPE_HEAT, session)
        return session

    def token_for_auth(self, auth):
        return self.get_keystone_session().get_auth_headers(auth).get(
            'X-Auth-Token')



# -*- coding: utf-8 -*-
# Copyright (c) 2014 Adrien Vergé <adrien.verge@numergy.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1 import exceptions as keystone_exceptions
import mock
from oslo_config import cfg

from sahara import main
from sahara.tests.unit import base as test_base
from sahara.utils.openstack import cinder


CONF = cfg.CONF


class TestCinder(test_base.SaharaTestCase):
    def setup_context(self, username="test_user", tenant_id="tenant_1",
                      token="test_auth_token", tenant_name='test_tenant',
                      **kwargs):
        self.override_config('os_region_name', 'RegionOne')

        # Fake service_catalog with both volumev2
        # and volumev3 services available
        service_catalog = '''[
            { "type": "volumev2",
              "endpoints": [ { "region": "RegionOne",
                               "internalURL": "http://localhost/" } ] },
            { "type": "volumev3",
              "endpoints": [ { "region": "RegionOne",
                               "internalURL": "http://localhost/" } ] }]'''

        super(TestCinder, self).setup_context(
            username=username, tenant_id=tenant_id, token=token,
            tenant_name=tenant_name, service_catalog=service_catalog, **kwargs)

    @mock.patch('sahara.utils.openstack.keystone.auth')
    @mock.patch('cinderclient.v3.client.Client')
    @mock.patch('cinderclient.v2.client.Client')
    def test_get_cinder_client_api_v2(self, patched2, patched3, auth):
        self.override_config('api_version', 2, group='cinder')
        patched2.return_value = FakeCinderClient(2)
        patched3.return_value = FakeCinderClient(3)

        client = cinder.client()
        self.assertEqual(2, client.client.api_version)

    @mock.patch('sahara.utils.openstack.keystone.auth')
    @mock.patch('cinderclient.v3.client.Client')
    @mock.patch('cinderclient.v2.client.Client')
    def test_get_cinder_client_api_v3(self, patched2, patched3, auth):
        self.override_config('api_version', 3, group='cinder')
        patched2.return_value = FakeCinderClient(2)
        patched3.return_value = FakeCinderClient(3)

        client = cinder.client()
        self.assertEqual(3, client.client.api_version)

    def test_cinder_bad_api_version(self):
        self.override_config('api_version', 1, group='cinder')
        cinder.validate_config()

        # Check bad version falls back to latest supported version
        self.assertEqual(3, main.CONF.cinder.api_version)

    @mock.patch('sahara.utils.openstack.base.url_for')
    def test_check_cinder_exists(self, mock_url_for):
        mock_url_for.return_value = None
        self.assertTrue(cinder.check_cinder_exists())

        mock_url_for.reset_mock()

        mock_url_for.side_effect = keystone_exceptions.EndpointNotFound()
        self.assertFalse(cinder.check_cinder_exists())


class FakeCinderClient(object):
    def __init__(self, api_version):
        class FakeCinderHTTPClient(object):
            def __init__(self, api_version):
                self.api_version = api_version
        self.client = FakeCinderHTTPClient(api_version)



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import base as pl_base
from sahara.plugins import provisioning as pr_base

SAMPLE_CLUSTER = {
    'plugin_name': 'fake',
    'hadoop_version': 'test_version',
    'tenant_id': 'tenant_1',
    'name': 'test_cluster',
    'user_keypair_id': 'my_keypair',
    'node_groups': [
        {
            'auto_security_group': True,
            'name': 'ng_1',
            'flavor_id': '42',
            'node_processes': ['p1', 'p2'],
            'count': 1
        },
        {
            'auto_security_group': False,
            'name': 'ng_2',
            'flavor_id': '42',
            'node_processes': ['p3', 'p4'],
            'count': 3
        },
        {
            'auto_security_group': False,
            'name': 'ng_3',
            'flavor_id': '42',
            'node_processes': ['p3', 'p4'],
            'count': 1
        }
    ],
    'cluster_configs': {
        'service_1': {
            'config_2': 'value_2'
        },
        'service_2': {
            'config_1': 'value_1'
        }
    },
}

SCALE_DATA = {
    'resize_node_groups': [
        {
            'name': 'ng_1',
            'count': 3,
        },
        {
            'name': 'ng_2',
            'count': 2,
        }
    ],
    'add_node_groups': [
        {
            'auto_security_group': True,
            'name': 'ng_4',
            'flavor_id': '42',
            'node_processes': ['p1', 'p2'],
            'count': 1
        },
    ]
}

SCALE_DATA_SPECIFIC_INSTANCE = {
    'resize_node_groups': [
        {
            'name': 'ng_1',
            'count': 3,
        },
        {
            'name': 'ng_2',
            'count': 1,
            'instances': ['ng_2_0']
        }
    ],
    'add_node_groups': []
}

SCALE_DATA_N_SPECIFIC_INSTANCE = {
    'resize_node_groups': [
        {
            'name': 'ng_1',
            'count': 3,
        },
        {
            'name': 'ng_2',
            'count': 1,
            'instances': ['ng_2_0', 'ng_2_2']
        }
    ],
    'add_node_groups': []
}


class FakePlugin(pr_base.ProvisioningPluginBase):
    _info = {}
    name = "fake"

    def __init__(self, calls_order):
        self.calls_order = calls_order

    def configure_cluster(self, cluster):
        pass

    def start_cluster(self, cluster):
        pass

    def get_description(self):
        return "Some description"

    def get_title(self):
        return "Fake plugin"

    def validate(self, cluster):
        self.calls_order.append('validate')

    def get_open_ports(self, node_group):
        self.calls_order.append('get_open_ports')

    def validate_scaling(self, cluster, to_be_enlarged, additional):
        self.calls_order.append('validate_scaling')

    def get_versions(self):
        return ['0.1', '0.2']

    def get_node_processes(self, version):
        return {'HDFS': ['namenode', 'datanode']}

    def get_configs(self, version):
        return []

    def recommend_configs(self, cluster, scaling=False):
        self.calls_order.append('recommend_configs')


class FakePluginManager(pl_base.PluginManager):
    def __init__(self, calls_order):
        super(FakePluginManager, self).__init__()
        self.plugins['fake'] = FakePlugin(calls_order)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re

from keystoneauth1 import identity as keystone_identity
from keystoneclient.v2_0 import client as keystone_client
from keystoneclient.v3 import client as keystone_client_v3
from oslo_config import cfg
from oslo_log import log as logging

from sahara import context
from sahara.service import sessions
from sahara.utils.openstack import base


LOG = logging.getLogger(__name__)


def _get_keystoneauth_cfg(name):
    """get the keystone auth cfg

    Fetch value of keystone_authtoken group from config file when not
    available as part of GroupAttr.
    :rtype: String
    :param name: property name to be retrieved
    """
    try:
        value_list = CONF._namespace._get_file_value([('keystone_authtoken',
                                                       name)])
        if isinstance(value_list, tuple):
            value_list = value_list[0]
        cfg_val = value_list[0]
        if name == "auth_url" and not re.findall(r'\/v[2-3].*', cfg_val):
            cfg_val += "/v3"
        return cfg_val
    except KeyError:
        if name in ["user_domain_name", "project_domain_name"]:
            return "Default"
        else:
            raise


def validate_config():
    if any(map(lambda o: getattr(CONF.trustee, o) is None, CONF.trustee)):
        for replace_opt in CONF.trustee:
            CONF.set_override(replace_opt, _get_keystoneauth_cfg(replace_opt),
                              group="trustee")
        LOG.warning("""
 __        __               _
 \ \      / /_ _ _ __ _ __ (_)_ __   __ _
  \ \ /\ / / _` | '__| '_ \| | '_ \ / _` |
   \ V  V / (_| | |  | | | | | | | | (_| |
    \_/\_/ \__,_|_|  |_| |_|_|_| |_|\__, |
                                    |___/

Using the [keystone_authtoken] user as the Sahara trustee user directly is
deprecated. Please add the trustee credentials you need to the [trustee]
section of your sahara.conf file.
    """)


opts = [
    # TODO(alazarev) Move to [keystone] section
    cfg.BoolOpt('use_identity_api_v3',
                default=True,
                help='Enables Sahara to use Keystone API v3. '
                     'If that flag is disabled, '
                     'per-job clusters will not be terminated '
                     'automatically.')
]

ssl_opts = [
    cfg.BoolOpt('api_insecure',
                default=False,
                help='Allow to perform insecure SSL requests to keystone.'),
    cfg.StrOpt('ca_file',
               help='Location of ca certificates file to use for keystone '
                    'client requests.'),
    cfg.StrOpt("endpoint_type",
               default="internalURL",
               help="Endpoint type for keystone client requests")
]

keystone_group = cfg.OptGroup(name='keystone',
                              title='Keystone client options')

trustee_opts = [
    cfg.StrOpt('username',
               help='Username for trusts creation'),
    cfg.StrOpt('password',
               help='Password for trusts creation'),
    cfg.StrOpt('project_name',
               help='Project name for trusts creation'),
    cfg.StrOpt('user_domain_name',
               help='User domain name for trusts creation',
               default="Default"),
    cfg.StrOpt('project_domain_name',
               help='Project domain name for trusts creation',
               default="Default"),
    cfg.StrOpt('auth_url',
               help='Auth url for trusts creation'),
]

trustee_group = cfg.OptGroup(name='trustee', title="Trustee options")

CONF = cfg.CONF
CONF.register_group(keystone_group)
CONF.register_group(trustee_group)
CONF.register_opts(opts)
CONF.register_opts(ssl_opts, group=keystone_group)
CONF.register_opts(trustee_opts, group=trustee_group)


def auth():
    '''Return a token auth plugin for the current context.'''
    ctx = context.current()
    return ctx.auth_plugin or token_auth(token=context.get_auth_token(),
                                         project_id=ctx.tenant_id)


def auth_for_admin(project_name=None, trust_id=None):
    '''Return an auth plugin for the admin.

    :param project_name: a project to scope the auth with (optional).

    :param trust_id: a trust to scope the auth with (optional).

    :returns: an auth plugin object for the admin.
    '''
    # TODO(elmiko) revisit the project_domain_name if we start getting
    # into federated authentication. it will need to match the domain that
    # the project_name exists in.
    auth = _password_auth(
        username=CONF.trustee.username,
        password=CONF.trustee.password,
        project_name=project_name,
        user_domain_name=CONF.trustee.user_domain_name,
        project_domain_name=CONF.trustee.project_domain_name,
        trust_id=trust_id)
    return auth


def auth_for_proxy(username, password, trust_id=None):
    '''Return an auth plugin for the proxy user.

    :param username: the name of the proxy user.

    :param password: the proxy user's password.

    :param trust_id: a trust to scope the auth with (optional).

    :returns: an auth plugin object for the proxy user.
    '''
    auth = _password_auth(
        username=username,
        password=password,
        user_domain_name=CONF.proxy_user_domain_name,
        trust_id=trust_id)
    return auth


def client():
    '''Return the current context client.'''
    return client_from_auth(auth())


def client_for_admin():
    '''Return the Sahara admin user client.'''
    auth = auth_for_admin(
        project_name=CONF.trustee.project_name)
    return client_from_auth(auth)


def client_from_auth(auth):
    '''Return a session based client from the auth plugin provided.

    A session is obtained from the global session cache.

    :param auth: the auth plugin object to use in client creation.

    :returns: a keystone client
    '''
    session = sessions.cache().get_session(sessions.SESSION_TYPE_KEYSTONE)
    if CONF.use_identity_api_v3:
        client_class = keystone_client_v3.Client
    else:
        client_class = keystone_client.Client
    return client_class(session=session, auth=auth)


def project_id_from_auth(auth):
    '''Return the project id associated with an auth plugin.

    :param auth: the auth plugin to inspect.

    :returns: the project id associated with the auth plugin.
    '''
    return auth.get_project_id(
        sessions.cache().get_session(sessions.SESSION_TYPE_KEYSTONE))


def service_catalog_from_auth(auth):
    '''Return the service catalog associated with an auth plugin.

    :param auth: the auth plugin to inspect.

    :returns: a list containing the service catalog.
    '''
    access_info = auth.get_access(
        sessions.cache().get_session(sessions.SESSION_TYPE_KEYSTONE))
    if access_info.has_service_catalog():
        return access_info.service_catalog.catalog
    else:
        return []


def token_auth(token, project_id=None, project_name=None,
               project_domain_name='Default'):
    '''Return a token auth plugin object.

    :param token: the token to use for authentication.

    :param project_id: the project(ex. tenant) id to scope the auth.

    :returns: a token auth plugin object.
    '''
    token_kwargs = dict(
        auth_url=base.retrieve_auth_url(CONF.keystone.endpoint_type),
        token=token
    )
    if CONF.use_identity_api_v3:
        token_kwargs.update(dict(
            project_id=project_id,
            project_name=project_name,
            project_domain_name=project_domain_name,
        ))
        auth = keystone_identity.v3.Token(**token_kwargs)
    else:
        token_kwargs.update(dict(
            tenant_id=project_id,
            tenant_name=project_name,
        ))
        auth = keystone_identity.v2.Token(**token_kwargs)
    return auth


def token_from_auth(auth):
    '''Return an authentication token from an auth plugin.

    :param auth: the auth plugin to acquire a token from.

    :returns: an auth token in string format.
    '''
    return sessions.cache().token_for_auth(auth)


def user_id_from_auth(auth):
    '''Return a user id associated with an auth plugin.

    :param auth: the auth plugin to inspect.

    :returns: a token associated with the auth.
    '''
    return auth.get_user_id(sessions.cache().get_session(
        sessions.SESSION_TYPE_KEYSTONE))


def _password_auth(username, password,
                   project_name=None, user_domain_name=None,
                   project_domain_name=None, trust_id=None):
    '''Return a password auth plugin object.

    :param username: the user to authenticate as.

    :param password: the user's password.

    :param project_name: the project(ex. tenant) name to scope the auth.

    :param user_domain_name: the domain the user belongs to.

    :param project_domain_name: the domain the project belongs to.

    :param trust_id: a trust id to scope the auth.

    :returns: a password auth plugin object.
    '''
    passwd_kwargs = dict(
        auth_url=CONF.trustee.auth_url,
        username=username,
        password=password
    )
    if CONF.use_identity_api_v3:
        passwd_kwargs.update(dict(
            project_name=project_name,
            user_domain_name=user_domain_name,
            project_domain_name=project_domain_name,
            trust_id=trust_id
        ))
        auth = keystone_identity.v3.Password(**passwd_kwargs)
    else:
        passwd_kwargs.update(dict(
            tenant_name=project_name,
            trust_id=trust_id
        ))
        auth = keystone_identity.v2.Password(**passwd_kwargs)
    return auth



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
import six
import six.moves.urllib.parse as urlparse
import swiftclient

import sahara.exceptions as ex
from sahara.i18n import _
from sahara.service.castellan import utils as key_manager
from sahara.service.edp.job_binaries.base import JobBinaryType
from sahara.swift import utils as su
from sahara.utils.openstack import swift as sw

CONF = cfg.CONF


class SwiftType(JobBinaryType):
    def copy_binary_to_cluster(self, job_binary, **kwargs):
        r = kwargs.pop('remote')

        dst = self._generate_valid_path(job_binary)
        raw = self.get_raw_data(job_binary, **kwargs)

        r.write_file_to(dst, raw)
        return dst

    def validate_job_location_format(self, url):
        url = urlparse.urlparse(url)
        return url.scheme == "swift" and url.hostname

    def validate(self, data, **kwargs):
        if not kwargs.get('job_binary_id', None):
            extra = data.get("extra", {})
            # Should not be checked during job binary update
            if (not extra.get("user") or not extra.get("password")) and (
                    not CONF.use_domain_for_proxy_users):
                raise ex.BadJobBinaryException()

    def get_raw_data(self, job_binary, **kwargs):

        self._validate_job_binary_url(job_binary)

        proxy_configs = kwargs.pop('proxy_configs', None)
        with_context = kwargs.pop('with_context', False)

        if not with_context:
            conn_kwargs = {}
            if proxy_configs:
                conn_kwargs.update(username=proxy_configs.get(
                                   'proxy_username'),
                                   password=key_manager.get_secret(
                                   proxy_configs.get('proxy_password')),
                                   trust_id=proxy_configs.get(
                                   'proxy_trust_id'))
            else:
                conn_kwargs.update(username=job_binary.extra.get('user'),
                                   password=key_manager.get_secret(
                                   job_binary.extra.get('password')))
            conn = sw.client(**conn_kwargs)
        else:
            conn = sw.client_from_token()

        raw = self._get_raw_data(job_binary, conn)
        return raw

    def _get_names_from_url(self, url):
        parse = six.moves.urllib.parse.urlparse(url)
        return (parse.netloc + parse.path).split('/', 1)

    def _get_raw_data(self, job_binary, conn):
        names = self._get_names_from_url(job_binary.url)
        container, obj = names

        # if container name has '.sahara' suffix we need to strip it
        if container.endswith(su.SWIFT_URL_SUFFIX):
            container = container[:-len(su.SWIFT_URL_SUFFIX)]

        try:
            # First check the size
            headers = conn.head_object(container, obj)
            total_KB = int(headers.get('content-length', 0)) / 1024.0
            if total_KB > CONF.job_binary_max_KB:
                raise ex.DataTooBigException(
                    round(total_KB, 1), CONF.job_binary_max_KB,
                    _("Size of swift object (%(size)sKB) is greater "
                      "than maximum (%(maximum)sKB)"))

            headers, body = conn.get_object(container, obj)
        except swiftclient.ClientException as e:
            raise ex.SwiftClientException(six.text_type(e))

        return body

    def _validate_job_binary_url(self, job_binary):

        if not (job_binary.url.startswith(su.SWIFT_INTERNAL_PREFIX)):
            # This should have been guaranteed already,
            # but we'll check just in case.
            raise ex.BadJobBinaryException(
                _("Url for binary in internal swift must start with %s")
                % su.SWIFT_INTERNAL_PREFIX)
        names = self._get_names_from_url(job_binary.url)
        if len(names) == 1:
            # a container has been requested, this is currently unsupported
            raise ex.BadJobBinaryException(
                _('Url for binary in internal swift must specify an '
                  'object not a container'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import itertools

from sahara.common.policies import base
from sahara.common.policies import cluster
from sahara.common.policies import cluster_template
from sahara.common.policies import cluster_templates
from sahara.common.policies import clusters
from sahara.common.policies import data_source
from sahara.common.policies import data_sources
from sahara.common.policies import image
from sahara.common.policies import images
from sahara.common.policies import job
from sahara.common.policies import job_binaries
from sahara.common.policies import job_binary
from sahara.common.policies import job_binary_internals
from sahara.common.policies import job_executions
from sahara.common.policies import job_template
from sahara.common.policies import job_type
from sahara.common.policies import job_types
from sahara.common.policies import jobs
from sahara.common.policies import node_group_template
from sahara.common.policies import node_group_templates
from sahara.common.policies import plugin
from sahara.common.policies import plugins


def list_rules():
    return itertools.chain(
        base.list_rules(),
        clusters.list_rules(),
        cluster_templates.list_rules(),
        data_sources.list_rules(),
        images.list_rules(),
        job_binaries.list_rules(),
        job_binary_internals.list_rules(),
        job_executions.list_rules(),
        job_types.list_rules(),
        jobs.list_rules(),
        node_group_templates.list_rules(),
        plugins.list_rules(),
        cluster.list_rules(),
        cluster_template.list_rules(),
        data_source.list_rules(),
        image.list_rules(),
        job_binary.list_rules(),
        job_type.list_rules(),
        job.list_rules(),
        node_group_template.list_rules(),
        plugin.list_rules(),
        job_template.list_rules()
    )



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_utils import uuidutils
import six.moves.urllib.parse as urlparse

from sahara import exceptions as ex
from sahara.i18n import _
from sahara.service.edp.data_sources.base import DataSourceType
from sahara.service.edp import job_utils
from sahara.service.edp.utils import shares as shares_service


class ManilaType(DataSourceType):
    def validate(self, data):
        self._validate_url(data['url'])

    def _validate_url(self, url):
        if len(url) == 0:
            raise ex.InvalidDataException(_("Manila url must not be empty"))
        url = urlparse.urlparse(url)
        if url.scheme != "manila":
            raise ex.InvalidDataException(_("Manila url scheme must be"
                                            " 'manila'"))
        if not uuidutils.is_uuid_like(url.netloc):
            raise ex.InvalidDataException(_("Manila url netloc must be a"
                                            " uuid"))
        if not url.path:
            raise ex.InvalidDataException(_("Manila url path must not be"
                                            " empty"))

    def _prepare_cluster(self, url, cluster):
        path = self._get_share_path(url, cluster.shares or [])
        if path is None:
            path = job_utils.mount_share_at_default_path(url,
                                                         cluster)
        return path

    def get_runtime_url(self, url, cluster):
        # TODO(mariannelm): currently the get_runtime_url method is responsible
        # for preparing the cluster for the manila job type which is not the
        # best approach. In order to make a prepare_cluster method for manila
        # the edp/job_utils.py resolve_data_source_reference function must be
        # refactored
        path = self._prepare_cluster(url, cluster)
        # This gets us the mount point, but we need a file:// scheme to
        # indicate a local filesystem path
        return "file://{path}".format(path=path)

    def _get_share_path(self, url, shares):
        return shares_service.get_share_path(url, shares)



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import oslo_messaging
import six
import testtools

from sahara import conductor as cond
from sahara import context
from sahara import exceptions as exc
from sahara.plugins import base as pl_base
from sahara.plugins import utils as u
from sahara.service import api as service_api
from sahara.service.api.v2 import clusters as api
from sahara.tests.unit import base
import sahara.tests.unit.service.api.v2.base as api_base
from sahara.utils import cluster as c_u

conductor = cond.API


class FakeOps(object):
    def __init__(self, calls_order):
        self.calls_order = calls_order

    def provision_cluster(self, id):
        self.calls_order.append('ops.provision_cluster')
        cluster = conductor.cluster_get(context.ctx(), id)
        target_count = {}
        for node_group in cluster.node_groups:
            target_count[node_group.id] = node_group.count

        for node_group in cluster.node_groups:
            conductor.node_group_update(context.ctx(),
                                        node_group, {"count": 0})

        for node_group in cluster.node_groups:
            for i in range(target_count[node_group.id]):
                inst = {
                    "instance_id": node_group.name + '_' + str(i),
                    "instance_name": node_group.name + '_' + str(i)
                }
                conductor.instance_add(context.ctx(), node_group, inst)
        conductor.cluster_update(
            context.ctx(), id, {'status': c_u.CLUSTER_STATUS_ACTIVE})

    def provision_scaled_cluster(self, id, to_be_enlarged,
                                 node_group_instance_map=None):
        self.calls_order.append('ops.provision_scaled_cluster')
        cluster = conductor.cluster_get(context.ctx(), id)

        # Set scaled to see difference between active and scaled
        for (ng, count) in six.iteritems(to_be_enlarged):
            instances_to_delete = []
            if node_group_instance_map:
                if ng in node_group_instance_map:
                    instances_to_delete = self._get_instance(
                        cluster, node_group_instance_map[ng])
            for instance in instances_to_delete:
                conductor.instance_remove(context.ctx(), instance)

            conductor.node_group_update(context.ctx(), ng, {'count': count})
        conductor.cluster_update(context.ctx(), id, {'status': 'Scaled'})

    def update_keypair(self, id):
        self.calls_order.append('ops.update_keypair')
        cluster = conductor.cluster_get(context.ctx(), id)
        keypair_name = cluster.user_keypair_id
        nova_p = mock.patch("sahara.utils.openstack.nova.client")
        nova = nova_p.start()
        key = nova.get_keypair(keypair_name)
        nodes = u.get_instances(cluster)
        for instance in nodes:
            remote = mock.Mock()
            remote.execute_command(
                "echo {keypair} >> ~/.ssh/authorized_keys".format(
                    keypair=key.public_key))
            remote.reset_mock()

    def terminate_cluster(self, id, force):
        self.calls_order.append('ops.terminate_cluster')

    def _get_instance(self, cluster, instances_to_delete):
        instances = []
        for node_group in cluster.node_groups:
            for instance in node_group.instances:
                if instance.instance_id in instances_to_delete:
                    instances.append(instance)
        return instances


class TestClusterApi(base.SaharaWithDbTestCase):
    def setUp(self):
        super(TestClusterApi, self).setUp()
        self.calls_order = []
        self.override_config('plugins', ['fake'])
        pl_base.PLUGINS = api_base.FakePluginManager(self.calls_order)
        service_api.setup_api(FakeOps(self.calls_order))
        oslo_messaging.notify.notifier.Notifier.info = mock.Mock()
        self.ctx = context.ctx()

    @mock.patch('sahara.service.quotas.check_cluster', return_value=None)
    def test_create_cluster_success(self, check_cluster):
        cluster = api.create_cluster(api_base.SAMPLE_CLUSTER)
        self.assertEqual(1, check_cluster.call_count)
        result_cluster = api.get_cluster(cluster.id)
        self.assertEqual(c_u.CLUSTER_STATUS_ACTIVE, result_cluster.status)
        expected_count = {
            'ng_1': 1,
            'ng_2': 3,
            'ng_3': 1,
        }
        ng_count = 0
        for ng in result_cluster.node_groups:
            self.assertEqual(expected_count[ng.name], ng.count)
            ng_count += 1
        self.assertEqual(3, ng_count)
        api.terminate_cluster(result_cluster.id)
        self.assertEqual(
            ['get_open_ports', 'recommend_configs', 'validate',
             'ops.provision_cluster',
             'ops.terminate_cluster'], self.calls_order)

    @mock.patch('sahara.service.quotas.check_cluster', return_value=None)
    def test_create_multiple_clusters_success(self, check_cluster):
        MULTIPLE_CLUSTERS = api_base.SAMPLE_CLUSTER.copy()
        MULTIPLE_CLUSTERS['count'] = 2
        clusters = api.create_multiple_clusters(MULTIPLE_CLUSTERS)
        self.assertEqual(2, check_cluster.call_count)
        result_cluster1 = api.get_cluster(
            clusters['clusters'][0]['cluster']['id'])
        result_cluster2 = api.get_cluster(
            clusters['clusters'][1]['cluster']['id'])
        self.assertEqual(c_u.CLUSTER_STATUS_ACTIVE, result_cluster1.status)
        self.assertEqual(c_u.CLUSTER_STATUS_ACTIVE, result_cluster2.status)
        expected_count = {
            'ng_1': 1,
            'ng_2': 3,
            'ng_3': 1,
        }
        ng_count = 0
        for ng in result_cluster1.node_groups:
            self.assertEqual(expected_count[ng.name], ng.count)
            ng_count += 1
        self.assertEqual(3, ng_count)
        api.terminate_cluster(result_cluster1.id)
        api.terminate_cluster(result_cluster2.id)
        self.assertEqual(
            ['get_open_ports', 'recommend_configs', 'validate',
             'ops.provision_cluster',
             'get_open_ports', 'recommend_configs', 'validate',
             'ops.provision_cluster',
             'ops.terminate_cluster',
             'ops.terminate_cluster'], self.calls_order)

    @mock.patch('sahara.service.quotas.check_cluster')
    def test_create_multiple_clusters_failed(self, check_cluster):
        MULTIPLE_CLUSTERS = api_base.SAMPLE_CLUSTER.copy()
        MULTIPLE_CLUSTERS['count'] = 2
        check_cluster.side_effect = exc.QuotaException(
            'resource', 'requested', 'available')
        with testtools.ExpectedException(exc.QuotaException):
            api.create_cluster(api_base.SAMPLE_CLUSTER)
        self.assertEqual('Error', api.get_clusters()[0].status)

    @mock.patch('sahara.service.quotas.check_cluster')
    def test_create_cluster_failed(self, check_cluster):
        check_cluster.side_effect = exc.QuotaException(
            'resource', 'requested', 'available')
        with testtools.ExpectedException(exc.QuotaException):
            api.create_cluster(api_base.SAMPLE_CLUSTER)
        self.assertEqual('Error', api.get_clusters()[0].status)

    @mock.patch('sahara.service.quotas.check_cluster', return_value=None)
    @mock.patch('sahara.service.quotas.check_scaling', return_value=None)
    def test_scale_cluster_success(self, check_scaling, check_cluster):
        cluster = api.create_cluster(api_base.SAMPLE_CLUSTER)
        cluster = api.get_cluster(cluster.id)
        api.scale_cluster(cluster.id, api_base.SCALE_DATA)
        result_cluster = api.get_cluster(cluster.id)
        self.assertEqual('Scaled', result_cluster.status)
        expected_count = {
            'ng_1': 3,
            'ng_2': 2,
            'ng_3': 1,
            'ng_4': 1,
        }
        ng_count = 0
        for ng in result_cluster.node_groups:
            self.assertEqual(expected_count[ng.name], ng.count)
            ng_count += 1
        self.assertEqual(4, ng_count)
        api.terminate_cluster(result_cluster.id)
        self.assertEqual(
            ['get_open_ports', 'recommend_configs', 'validate',
             'ops.provision_cluster', 'get_open_ports', 'get_open_ports',
             'recommend_configs', 'validate_scaling',
             'ops.provision_scaled_cluster',
             'ops.terminate_cluster'], self.calls_order)

    @mock.patch('sahara.service.quotas.check_cluster', return_value=None)
    @mock.patch('sahara.service.quotas.check_scaling', return_value=None)
    def test_scale_cluster_n_specific_instances_success(self, check_scaling,
                                                        check_cluster):
        cluster = api.create_cluster(api_base.SAMPLE_CLUSTER)
        cluster = api.get_cluster(cluster.id)
        api.scale_cluster(cluster.id, api_base.SCALE_DATA_N_SPECIFIC_INSTANCE)
        result_cluster = api.get_cluster(cluster.id)
        self.assertEqual('Scaled', result_cluster.status)
        expected_count = {
            'ng_1': 3,
            'ng_2': 1,
            'ng_3': 1,
        }
        ng_count = 0
        for ng in result_cluster.node_groups:
            self.assertEqual(expected_count[ng.name], ng.count)
            ng_count += 1
        self.assertEqual(1, result_cluster.node_groups[1].count)
        self.assertNotIn('ng_2_0',
                         self._get_instances_ids(
                             result_cluster.node_groups[1]))
        self.assertNotIn('ng_2_2',
                         self._get_instances_ids(
                             result_cluster.node_groups[1]))
        self.assertEqual(3, ng_count)
        api.terminate_cluster(result_cluster.id)
        self.assertEqual(
            ['get_open_ports', 'recommend_configs', 'validate',
             'ops.provision_cluster', 'get_open_ports',
             'recommend_configs', 'validate_scaling',
             'ops.provision_scaled_cluster',
             'ops.terminate_cluster'], self.calls_order)

    @mock.patch('sahara.service.quotas.check_cluster', return_value=None)
    @mock.patch('sahara.service.quotas.check_scaling', return_value=None)
    def test_scale_cluster_specific_and_non_specific(self, check_scaling,
                                                     check_cluster):
        cluster = api.create_cluster(api_base.SAMPLE_CLUSTER)
        cluster = api.get_cluster(cluster.id)
        api.scale_cluster(cluster.id, api_base.SCALE_DATA_SPECIFIC_INSTANCE)
        result_cluster = api.get_cluster(cluster.id)
        self.assertEqual('Scaled', result_cluster.status)
        expected_count = {
            'ng_1': 3,
            'ng_2': 1,
            'ng_3': 1,
        }
        ng_count = 0
        for ng in result_cluster.node_groups:
            self.assertEqual(expected_count[ng.name], ng.count)
            ng_count += 1
        self.assertEqual(1, result_cluster.node_groups[1].count)
        self.assertNotIn('ng_2_0',
                         self._get_instances_ids(
                             result_cluster.node_groups[1]))
        self.assertEqual(3, ng_count)
        api.terminate_cluster(result_cluster.id)
        self.assertEqual(
            ['get_open_ports', 'recommend_configs', 'validate',
             'ops.provision_cluster', 'get_open_ports',
             'recommend_configs', 'validate_scaling',
             'ops.provision_scaled_cluster',
             'ops.terminate_cluster'], self.calls_order)

    def _get_instances_ids(self, node_group):
        instance_ids = []
        for instance in node_group.instances:
            instance_ids.append(instance.instance_id)
        return instance_ids

    @mock.patch('sahara.service.quotas.check_cluster', return_value=None)
    @mock.patch('sahara.service.quotas.check_scaling', return_value=None)
    def test_scale_cluster_failed(self, check_scaling, check_cluster):
        cluster = api.create_cluster(api_base.SAMPLE_CLUSTER)
        check_scaling.side_effect = exc.QuotaException(
            'resource', 'requested', 'available')
        with testtools.ExpectedException(exc.QuotaException):
            api.scale_cluster(cluster.id, {})

    def test_cluster_update(self):
        with mock.patch('sahara.service.quotas.check_cluster'):
            cluster = api.create_cluster(api_base.SAMPLE_CLUSTER)
            updated_cluster = api.update_cluster(
                cluster.id, {'description': 'Cluster'})
            self.assertEqual('Cluster', updated_cluster.description)

    def test_cluster_keypair_update(self):
        with mock.patch('sahara.service.quotas.check_cluster'):
            cluster = api.create_cluster(api_base.SAMPLE_CLUSTER)
            api.update_cluster(cluster.id, {'update_keypair': True})



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from sahara import exceptions
from sahara.service import validation
from sahara.tests.unit import base


class TestValidation(base.SaharaTestCase):
    def test_get_path(self):
        self.assertEqual('', validation._get_path([]))
        self.assertEqual('param: ', validation._get_path(['param']))
        self.assertEqual(
            'param[0][id]: ', validation._get_path(['param', 0, 'id']))

    def test_generate_error(self):
        # no errors occurred during validation
        self.assertIsNone(validation._generate_error([]))

        # one error occurred during validation
        err = mock.Mock(message='some error', path=['param'])
        generated_err = validation._generate_error([err])
        self.assertIsInstance(generated_err, exceptions.SaharaException)
        self.assertEqual('VALIDATION_ERROR', generated_err.code)
        self.assertIn('param: some error', generated_err.message)

        # several errors occurred during validation
        errors = [mock.Mock(message='first error', path=['param1']),
                  mock.Mock(
                      message='second error', path=['param2', 0, 'subparam'])]
        generated_err = validation._generate_error(errors)
        self.assertIsInstance(generated_err, exceptions.SaharaException)
        self.assertEqual('VALIDATION_ERROR', generated_err.code)
        self.assertIn('param1: first error\nparam2[0][subparam]: second error',
                      generated_err.message)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

import sahara.exceptions as ex
from sahara.service.health import verification_base
import sahara.service.validations.cluster_template_schema as ct_schema
from sahara.service.validations import shares


def _build_node_groups_schema():
    schema = copy.deepcopy(ct_schema.CLUSTER_TEMPLATE_SCHEMA)
    return schema['properties']['node_groups']


def _build_cluster_schema(api_version='v1'):
    if api_version == 'v1':
        cluster_schema = copy.deepcopy(ct_schema.CLUSTER_TEMPLATE_SCHEMA)
    elif api_version == 'v2':
        cluster_schema = copy.deepcopy(ct_schema.CLUSTER_TEMPLATE_SCHEMA_V2)
    else:
        raise ex.InvalidDataException('Invalid API version %s' % api_version)

    cluster_schema['properties'].update({
        "is_transient": {
            "type": "boolean"
        },
        "user_keypair_id": {
            "type": "string",
            "format": "valid_keypair_name",
        },
        "cluster_template_id": {
            "type": "string",
            "format": "uuid",
        }})

    if api_version == 'v2':
        cluster_schema['properties'].update({
            "count": {
                "type": "integer"
            }})
    return cluster_schema


CLUSTER_SCHEMA = _build_cluster_schema()
CLUSTER_SCHEMA_V2 = _build_cluster_schema('v2')

MULTIPLE_CLUSTER_SCHEMA = copy.deepcopy(CLUSTER_SCHEMA)
MULTIPLE_CLUSTER_SCHEMA['properties'].update({
    "count": {
        "type": "integer"
    }})
MULTIPLE_CLUSTER_SCHEMA['required'].append('count')

CLUSTER_UPDATE_SCHEMA = {
    "type": "object",
    "properties": {
        "description": {
            "type": ["string", "null"]
        },
        "name": {
            "type": "string",
            "minLength": 1,
            "maxLength": 50,
            "format": "valid_name_hostname",
        },
        "is_public": {
            "type": ["boolean", "null"],
        },
        "is_protected": {
            "type": ["boolean", "null"],
        },
        "verification": {
            "type": "object",
            "properties": {
                "status": {
                    "enum": verification_base.get_possible_ops(),
                }
            },
        },
        "shares": copy.deepcopy(shares.SHARE_SCHEMA),
    },
    "additionalProperties": False,
    "required": []
}
CLUSTER_UPDATE_SCHEMA_V2 = copy.deepcopy(CLUSTER_UPDATE_SCHEMA)
CLUSTER_UPDATE_SCHEMA_V2['properties'].update({
    "update_keypair": {
        "type": ["boolean", "null"]
    }})

CLUSTER_SCALING_SCHEMA = {
    "type": "object",
    "properties": {
        "resize_node_groups": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                    },
                    "count": {
                        "type": "integer",
                        "minimum": 0,
                    },
                },
                "additionalProperties": False,
                "required": [
                    "name",
                    "count",
                ]
            },
            "minItems": 1
        },
        "add_node_groups": _build_node_groups_schema(),
    },
    "additionalProperties": False,
    "anyOf": [
        {
            "required": ["resize_node_groups"]
        },
        {
            "required": ["add_node_groups"]
        }
    ]
}

CLUSTER_SCALING_SCHEMA_V2 = copy.deepcopy(CLUSTER_SCALING_SCHEMA)
CLUSTER_SCALING_SCHEMA_V2['properties']['resize_node_groups'][
    'items']['properties'].update(
        {
            "instances": {
                "type": "array",
                "items": {
                    "type": "string",
                },
            }
        })

CLUSTER_DELETE_SCHEMA_V2 = {
    "type": "object",
    "properties": {
        "force": {
            "type": "boolean"
        }
    },
    "additionalProperties": False,
    "required": []
}



# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.api import acl
from sahara.service.api.v2 import images as api
from sahara.service import validation as v
from sahara.service.validations import images as v_images
import sahara.utils.api as u


rest = u.RestV2('images', __name__)


@rest.get('/images')
@acl.enforce("data-processing:image:list")
@v.validate_request_params(['name', 'tags', 'username'])
def images_list():
    tags = u.get_request_args().getlist('tags')
    name = u.get_request_args().get('name', None)
    return u.render(images=[i.dict for i in api.get_images(name, tags)])


@rest.get('/images/<image_id>')
@acl.enforce("data-processing:image:get")
@v.check_exists(api.get_image, id='image_id')
@v.validate_request_params([])
def images_get(image_id):
    return u.render(api.get_registered_image(id=image_id).wrapped_dict)


@rest.post('/images/<image_id>')
@acl.enforce("data-processing:image:register")
@v.check_exists(api.get_image, id='image_id')
@v.validate(v_images.image_register_schema, v_images.check_image_register)
@v.validate_request_params([])
def images_set(image_id, data):
    return u.render(api.register_image(image_id, **data).wrapped_dict)


@rest.delete('/images/<image_id>')
@acl.enforce("data-processing:image:unregister")
@v.check_exists(api.get_image, id='image_id')
@v.validate_request_params([])
def images_unset(image_id):
    api.unregister_image(image_id)
    return u.render()


@rest.get('/images/<image_id>/tags')
@acl.enforce("data-processing:image:get-tags")
@v.check_exists(api.get_image, id='image_id')
@v.validate_request_params([])
def image_tags_get(image_id):
    return u.render(api.get_image_tags(image_id))


@rest.put('/images/<image_id>/tags', status_code=200)
@acl.enforce("data-processing:image:set-tags")
@v.check_exists(api.get_image, id='image_id')
@v.validate(v_images.image_tags_schema, v_images.check_tags)
@v.validate_request_params([])
def image_tags_update(image_id, data):
    return u.render(api.set_image_tags(image_id, **data).wrapped_dict)


@rest.delete('/images/<image_id>/tags')
@acl.enforce("data-processing:image:remove-tags")
@v.check_exists(api.get_image, id='image_id')
@v.validate_request_params([])
def image_tags_delete(image_id):
    api.remove_image_tags(image_id)
    return u.render()



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from sahara.tests.unit import base as testbase
from sahara.utils.openstack import swift


class SwiftClientTest(testbase.SaharaTestCase):

    @mock.patch('sahara.swift.swift_helper.retrieve_tenant')
    @mock.patch('sahara.swift.utils.retrieve_auth_url')
    @mock.patch('swiftclient.Connection')
    def test_client(self, swift_connection, retrieve_auth_url,
                    retrieve_tenant):
        swift.client('testuser', '12345')
        self.assertEqual(1, swift_connection.call_count)

    @mock.patch('sahara.utils.openstack.base.url_for')
    @mock.patch('swiftclient.Connection')
    @mock.patch('sahara.utils.openstack.keystone.token_from_auth')
    @mock.patch('sahara.utils.openstack.keystone.auth_for_proxy')
    def test_client_with_trust(self, auth_for_proxy, token_from_auth,
                               swift_connection, url_for):
        swift.client('testuser', '12345', 'test_trust')
        self.assertEqual(1, auth_for_proxy.call_count)
        self.assertEqual(1, token_from_auth.call_count)
        self.assertEqual(1, swift_connection.call_count)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import itertools

from oslo_utils import netutils
from six.moves.urllib import parse as urlparse

from sahara.i18n import _
from sahara.plugins import base as plugins_base
from sahara.plugins import exceptions as ex
from sahara.utils import api_validator
from sahara.utils import cluster as cluster_utils
from sahara.utils import cluster_progress_ops as ops
from sahara.utils import configs as sahara_configs
from sahara.utils import crypto
from sahara.utils import files
from sahara.utils import general
from sahara.utils.openstack import nova
from sahara.utils import poll_utils
from sahara.utils import proxy
from sahara.utils import remote
from sahara.utils import rpc
from sahara.utils import types
from sahara.utils import xmlutils


event_wrapper = ops.event_wrapper


def get_node_groups(cluster, node_process=None, **kwargs):
    return [ng for ng in cluster.node_groups
            if (node_process is None or
                node_process in ng.node_processes)]


def get_instances_count(cluster, node_process=None, **kwargs):
    return sum([ng.count for ng in get_node_groups(cluster, node_process)])


def get_instances(cluster, node_process=None, **kwargs):
    nodes = get_node_groups(cluster, node_process)
    return list(itertools.chain(*[node.instances for node in nodes]))


def get_instance(cluster, node_process, **kwargs):
    instances = get_instances(cluster, node_process)
    if len(instances) > 1:
        raise ex.InvalidComponentCountException(
            node_process, _('0 or 1'), len(instances))
    return instances[0] if instances else None


def generate_host_names(nodes, **kwargs):
    return "\n".join([n.hostname() for n in nodes])


def generate_fqdn_host_names(nodes, **kwargs):
    return "\n".join([n.fqdn() for n in nodes])


def get_port_from_address(address, **kwargs):
    parse_result = urlparse.urlparse(address)
    # urlparse do not parse values like 0.0.0.0:8000,
    # netutils do not parse values like http://localhost:8000,
    # so combine approach is using
    if parse_result.port:
        return parse_result.port
    else:
        return netutils.parse_host_port(address)[1]


def instances_with_services(instances, node_processes, **kwargs):
    node_processes = set(node_processes)
    return list(filter(
        lambda x: node_processes.intersection(
            x.node_group.node_processes), instances))


def start_process_event_message(process, **kwargs):
    return _("Start the following process(es): {process}").format(
        process=process)


def get_config_value_or_default(
        service=None, name=None, cluster=None, config=None, **kwargs):
    if not config:
        if not service or not name:
            raise RuntimeError(_("Unable to retrieve config details"))
        default_value = None
    else:
        service = config.applicable_target
        name = config.name
        default_value = config.default_value

    cluster_configs = cluster.cluster_configs
    if cluster_configs.get(service, {}).get(name, None) is not None:
        return cluster_configs.get(service, {}).get(name, None)

    # Try getting config from the cluster.
    for ng in cluster.node_groups:
        if (ng.configuration().get(service) and
                ng.configuration()[service].get(name)):
            return ng.configuration()[service][name]

    # Find and return the default
    if default_value is not None:
        return default_value

    plugin = plugins_base.PLUGINS.get_plugin(cluster.plugin_name)
    configs = plugin.get_all_configs(cluster.hadoop_version)

    for config in configs:
        if config.applicable_target == service and config.name == name:
            return config.default_value

    raise RuntimeError(_("Unable to get parameter '%(param_name)s' from "
                         "service %(service)s"),
                       {'param_name': name, 'service': service})


def cluster_get_instances(cluster, instances_ids=None, **kwargs):
    return cluster_utils.get_instances(cluster, instances_ids)


def check_cluster_exists(cluster, **kwargs):
    return cluster_utils.check_cluster_exists(cluster)


def add_provisioning_step(cluster_id, step_name, total, **kwargs):
    return ops.add_provisioning_step(cluster_id, step_name, total)


def add_successful_event(instance, **kwargs):
    ops.add_successful_event(instance)


def add_fail_event(instance, exception, **kwargs):
    ops.add_fail_event(instance, exception)


def merge_configs(config_a, config_b, **kwargs):
    return sahara_configs.merge_configs(config_a, config_b)


def generate_key_pair(key_length=2048, **kwargs):
    return crypto.generate_key_pair(key_length)


def get_file_text(file_name, package='sahara', **kwargs):
    return files.get_file_text(file_name, package)


def try_get_file_text(file_name, package='sahara', **kwargs):
    return files.try_get_file_text(file_name, package)


def get_by_id(lst, id, **kwargs):
    return general.get_by_id(lst, id)


def natural_sort_key(s, **kwargs):
    return general.natural_sort_key(s)


def get_flavor(**kwargs):
    return nova.get_flavor(**kwargs)


def poll(get_status, kwargs=None, args=None, operation_name=None,
         timeout_name=None, timeout=poll_utils.DEFAULT_TIMEOUT,
         sleep=poll_utils.DEFAULT_SLEEP_TIME, exception_strategy='raise'):
    poll_utils.poll(get_status, kwargs=kwargs, args=args,
                    operation_name=operation_name,
                    timeout_name=timeout_name, timeout=timeout,
                    sleep=sleep, exception_strategy=exception_strategy)


def plugin_option_poll(cluster, get_status, option, operation_name,
                       sleep_time, kwargs):
    poll_utils.plugin_option_poll(cluster, get_status, option,
                                  operation_name, sleep_time, kwargs)


def create_proxy_user_for_cluster(cluster, **kwargs):
    return proxy.create_proxy_user_for_cluster(cluster)


def get_remote(instance, **kwargs):
    return remote.get_remote(instance)


def rpc_setup(service_name, **kwargs):
    rpc.setup(service_name)


def transform_to_num(s, **kwargs):
    return types.transform_to_num(s)


def is_int(s, **kwargs):
    return types.is_int(s)


def parse_hadoop_xml_with_name_and_value(data, **kwargs):
    return xmlutils.parse_hadoop_xml_with_name_and_value(data)


def create_hadoop_xml(configs, config_filter=None, **kwargs):
    return xmlutils.create_hadoop_xml(configs, config_filter)


def create_elements_xml(configs, **kwargs):
    return xmlutils.create_elements_xml(configs)


def load_hadoop_xml_defaults(file_name, package, **kwargs):
    return xmlutils.load_hadoop_xml_defaults(file_name, package)


def get_property_dict(elem, **kwargs):
    return xmlutils.get_property_dict(elem)


class PluginsApiValidator(api_validator.ApiValidator):
    def __init__(self, schema, **kwargs):
        super(PluginsApiValidator, self).__init__(schema)



# Copyright (c) 2015 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

JOB_BINARY_SCHEMA = {
    "type": "object",
    "properties": {
        "name": {
            "type": "string",
            "minLength": 1,
            "maxLength": 50,
            "format": "valid_name"
        },
        "description": {
            "type": "string"
        },
        "url": {
            "type": "string",
            "format": "valid_job_location"
        },
        "is_public": {
            "type": ["boolean", "null"],
        },
        "is_protected": {
            "type": ["boolean", "null"],
        },
        # extra is simple_config for now because we may need not only
        # user-password it the case of external storage
        "extra": {
            "type": "simple_config",
        }
    },
    "additionalProperties": False,
    "required": [
        "name",
        "url"
    ]
}

JOB_BINARY_UPDATE_SCHEMA = copy.copy(JOB_BINARY_SCHEMA)
JOB_BINARY_UPDATE_SCHEMA["required"] = []



# Copyright 2014 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""placeholder

Revision ID: 004
Revises: 003
Create Date: 2014-04-01 21:04:57.627883

"""

# revision identifiers, used by Alembic.
revision = '004'
down_revision = '003'


def upgrade():
    pass



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_policy import policy

from sahara.common.policies import base


images_policies = [
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_IMAGES % 'add_tags',
        check_str=base.UNPROTECTED,
        description='Add tags to image.',
        operations=[{'path': '/v1.1/{project_id}/images/{image_id}/tag',
                     'method': 'POST'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_IMAGES % 'register',
        check_str=base.UNPROTECTED,
        description='Register image.',
        operations=[{'path': '/v1.1/{project_id}/images/{image_id}',
                     'method': 'POST'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_IMAGES % 'get_all',
        check_str=base.UNPROTECTED,
        description='List images.',
        operations=[{'path': '/v1.1/{project_id}/images', 'method': 'GET'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_IMAGES % 'unregister',
        check_str=base.UNPROTECTED,
        description='Unregister image.',
        operations=[{'path': '/v1.1/{project_id}/images/{image_id}',
                     'method': 'POST'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_IMAGES % 'get',
        check_str=base.UNPROTECTED,
        description='Show image details.',
        operations=[{'path': '/v1.1/{project_id}/images/{image_id}',
                     'method': 'GET'}]),
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_IMAGES % 'remove_tags',
        check_str=base.UNPROTECTED,
        description='Remove tags from image.',
        operations=[{'path': '/v1.1/{project_id}/images/{image_id}/untag',
                     'method': 'POST'}]),
]


def list_rules():
    return images_policies



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Cluster creation related checks"""

from sahara import conductor as c
from sahara import context
from sahara import exceptions as ex
from sahara.i18n import _

conductor = c.API

data_source_type = {
    "type": "string",
    "enum": ["swift", "hdfs", "maprfs", "manila", "s3"]
}

job_configs = {
    "type": "object",
    "properties": {
        "configs": {
            "type": "simple_config",
        },
        "params": {
            "type": "simple_config",
        },
        "args": {
            "type": "array",
            "items": {
                "type": "string"
            }
        },
        "job_execution_info": {
            "type": "simple_config",
        }
    },
    "additionalProperties": False,
}


def check_data_source_unique_name(ds_name):
    if ds_name in [ds.name for ds in
                   conductor.data_source_get_all(context.ctx(),
                                                 name=ds_name)]:
        raise ex.NameAlreadyExistsException(
            _("Data source with name '%s' "
              "already exists") % ds_name)


def check_data_source_exists(data_source_id):
    if not conductor.data_source_get(context.ctx(), data_source_id):
        raise ex.InvalidReferenceException(
            _("DataSource with id '%s' doesn't exist") % data_source_id)


def check_job_unique_name(job_name):
    if job_name in [job.name for job in
                    conductor.job_get_all(context.ctx(),
                                          name=job_name)]:
        raise ex.NameAlreadyExistsException(_("Job with name '%s' "
                                              "already exists") % job_name)


def check_job_binary_internal_exists(jbi_id):
    if not conductor.job_binary_internal_get(context.ctx(), jbi_id):
        raise ex.InvalidReferenceException(
            _("JobBinaryInternal with id '%s' doesn't exist") % jbi_id)


def check_data_sources_are_different(data_source_1_id, data_source_2_id):
    ds1 = conductor.data_source_get(context.ctx(), data_source_1_id)
    ds2 = conductor.data_source_get(context.ctx(), data_source_2_id)

    if ds1.type == ds2.type and ds1.url == ds2.url:
        raise ex.InvalidDataException(_('Provided input and output '
                                        'DataSources reference the same '
                                        'location: %s') % ds1.url)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import _io
import pickle  # nosec
import sys
import traceback

from oslo_utils import reflection


def main():
    # NOTE(dmitryme): since we do not read stderr in the main process,
    # we need to flush it somewhere, otherwise both processes might
    # hang because of i/o buffer overflow.
    with open('/dev/null', 'w') as sys.stderr:
        while True:
            result = dict()

            try:
                # TODO(elmiko) these pickle usages should be
                # reinvestigated to determine a more secure manner to
                # deploy remote commands.
                if isinstance(sys.stdin, _io.TextIOWrapper):
                    func = pickle.load(sys.stdin.buffer)  # nosec
                    args = pickle.load(sys.stdin.buffer)  # nosec
                    kwargs = pickle.load(sys.stdin.buffer)  # nosec
                else:
                    func = pickle.load(sys.stdin)  # nosec
                    args = pickle.load(sys.stdin)  # nosec
                    kwargs = pickle.load(sys.stdin)  # nosec

                result['output'] = func(*args, **kwargs)
            except BaseException as e:
                cls_name = reflection.get_class_name(e, fully_qualified=False)
                result['exception'] = cls_name + ': ' + str(e)
                result['traceback'] = traceback.format_exc()

            if isinstance(sys.stdin, _io.TextIOWrapper):
                pickle.dump(result, sys.stdout.buffer, protocol=2)  # nosec
            else:
                pickle.dump(result, sys.stdout, protocol=2)  # nosec
            sys.stdout.flush()



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from sahara.cli import sahara_all
from sahara.cli import sahara_api
from sahara.cli import sahara_engine
from sahara.tests.unit import base


class TestSaharaCLI(base.SaharaTestCase):

    def setUp(self):
        super(TestSaharaCLI, self).setUp()

        modules = [
            'sahara.main.setup_common',
            'oslo_service.wsgi.Server.__init__',
            'oslo_service.wsgi.Loader'
        ]

        self.patchers = []
        for module in modules:
            patch = mock.patch(module)
            patch.start()
            self.patchers.append(patch)

        mock_get_pl_patch = mock.patch('sahara.main.get_process_launcher')
        self.patchers.append(mock_get_pl_patch)
        self.mock_get_pl = mock_get_pl_patch.start()

        mock_start_server_patch = mock.patch(
            'sahara.main.SaharaWSGIService.start')
        self.patchers.append(mock_start_server_patch)
        self.mock_start_server = mock_start_server_patch.start()

    def tearDown(self):
        super(TestSaharaCLI, self).tearDown()
        for patcher in reversed(self.patchers):
            patcher.stop()

    @mock.patch('sahara.main.setup_sahara_api')
    def test_main_start_api(self, mock_setup_sahara_api):

        sahara_api.main()

        self.mock_start_server.assert_called_once_with()
        self.mock_get_pl.return_value.wait.assert_called_once_with()

    @mock.patch('sahara.utils.rpc.RPCServer.get_service')
    @mock.patch('oslo_service.service.ProcessLauncher')
    @mock.patch('sahara.main._get_ops_driver')
    @mock.patch('sahara.service.ops.OpsServer')
    def test_main_start_engine(self, mock_ops_server, mock_get_ops_driver,
                               mock_pl, mock_get_service):

        self.mock_get_pl.return_value = mock_pl
        mock_ops_server.return_value.get_service.return_value = (
            mock_get_service)

        sahara_engine.main()

        mock_pl.launch_service.assert_called_once_with(mock_get_service)
        mock_pl.wait.assert_called_once_with()

    def test_main_start_all(self):

        sahara_all.main()

        self.mock_start_server.assert_called_once_with()
        self.mock_get_pl.return_value.wait.assert_called_once_with()



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from sahara import exceptions as exc
from sahara.tests.unit import base


class TestExceptions(base.SaharaTestCase):
    def _validate_exc(self, exc, expected_message, *args, **kwargs):
        message = ""
        try:
            raise exc(*args, **kwargs)
        except exc as e:
            message = six.text_type(e)
            if message.find("\nError ID") != -1:
                message = message.split("\nError ID")[0]

        self.assertEqual(expected_message, message)

    def test_not_found(self):
        self._validate_exc(
            exc.NotFoundException, "Object 'sample' is not found", "sample")
        self._validate_exc(
            exc.NotFoundException, "Not found 'sample'", "sample",
            "Not found '%s'")

    def test_name_already_exists(self):
        self._validate_exc(
            exc.NameAlreadyExistsException, "Name already exists")
        self._validate_exc(
            exc.NameAlreadyExistsException, "message", "message")

    def test_invalid_credentials(self):
        self._validate_exc(
            exc.InvalidCredentials, "Invalid credentials")
        self._validate_exc(
            exc.InvalidCredentials, "message", "message")

    def test_invalid_reference(self):
        self._validate_exc(
            exc.InvalidReferenceException, "Invalid object reference")
        self._validate_exc(
            exc.InvalidReferenceException, "message", "message")

    def test_remote_command(self):
        exc_message = ('Error during command execution: "cmd"\n'
                       'Return code: 0\n'
                       'STDERR:\n'
                       'stderr\n'
                       'STDOUT:\n'
                       'stdout')
        self._validate_exc(
            exc.RemoteCommandException, exc_message, "cmd", "0",
            "stdout", "stderr")

    def test_invalid_data(self):
        self._validate_exc(
            exc.InvalidDataException, "Data is invalid")
        self._validate_exc(
            exc.InvalidDataException, "message", "message")

    def test_bad_job_binary_internal(self):
        ex_message = ("Job binary internal data must be a string of length "
                      "greater than zero")
        self._validate_exc(
            exc.BadJobBinaryInternalException, ex_message)
        self._validate_exc(
            exc.BadJobBinaryInternalException, "message", "message")

    def test_bad_job_binary(self):
        ex_message = ("To work with JobBinary located in internal swift"
                      " add 'user' and 'password' to extra")
        self._validate_exc(
            exc.BadJobBinaryException, ex_message)
        self._validate_exc(
            exc.BadJobBinaryException, "message", "message")

    def test_db_duplicate(self):
        ex_message = "Database object already exists"
        self._validate_exc(
            exc.DBDuplicateEntry, ex_message)
        self._validate_exc(
            exc.DBDuplicateEntry, "message", "message")

    def test_creation_failed(self):
        ex_message = "Object was not created"
        self._validate_exc(
            exc.CreationFailed, ex_message)
        self._validate_exc(
            exc.CreationFailed, "message", "message")

    def test_canceling_failed(self):
        ex_message = "Operation was not canceled"
        self._validate_exc(
            exc.CancelingFailed, ex_message)
        self._validate_exc(
            exc.CancelingFailed, "message", "message")

    def test_deletion_failed(self):
        ex_message = "Object was not deleted"
        self._validate_exc(
            exc.DeletionFailed, ex_message)
        self._validate_exc(
            exc.DeletionFailed, "message", "message")

    def test_missing_floating_network(self):
        message = ("Node Group name is missing 'floating_ip_pool' "
                   "field")
        self._validate_exc(
            exc.MissingFloatingNetworkException, message, "name")

    def test_swift_client(self):
        self._validate_exc(exc.SwiftClientException, "message", "message")

    def test_data_too_big(self):
        exc_message = ("Size of data (size) is greater than maximum "
                       "(maximum)")
        self._validate_exc(
            exc.DataTooBigException, "size, maximum", "size", "maximum",
            "%(size)s, %(maximum)s")
        self._validate_exc(
            exc.DataTooBigException, exc_message, "size", "maximum")

    def test_not_implemented(self):
        self._validate_exc(
            exc.NotImplementedException, "Feature 'bond' is not implemented",
            "bond")
        self._validate_exc(
            exc.NotImplementedException, "feature", "feature", "%s")

    def test_incorrect_state(self):
        self._validate_exc(exc.IncorrectStateError, "message", "message")

    def test_deprecated(self):
        self._validate_exc(exc.DeprecatedException, "message", "message")

    def test_forbidden(self):
        self._validate_exc(exc.Forbidden, "message", "message")
        self._validate_exc(exc.Forbidden, "You are not authorized "
                                          "to complete this action")

    def test_image_not_registered(self):
        self._validate_exc(
            exc.ImageNotRegistered, 'Image image is not registered in Sahara',
            'image')

    def test_malformed_request_body(self):
        self._validate_exc(
            exc.MalformedRequestBody, "Malformed message body: reason",
            "reason")

    def test_update_failed(self):
        self._validate_exc(
            exc.UpdateFailedException, "Object 'value' could not be updated",
            "value")
        self._validate_exc(
            exc.UpdateFailedException, "value", "value", "%s")



# Copyright 2010-2011 OpenStack Foundation
# Copyright 2012-2013 IBM Corp.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
#
# Ripped off from Nova's test_migrations.py
# The only difference between Nova and this code is usage of alembic instead
# of sqlalchemy migrations.
#
# There is an ongoing work to extact similar code to oslo incubator. Once it is
# extracted we'll be able to remove this file and use oslo.

import io
import os

import alembic
from alembic import command
from alembic import config as alembic_config
from alembic import migration
from alembic import script as alembic_script
from oslo_config import cfg
from oslo_db.sqlalchemy import test_migrations as t_m
from oslo_log import log as logging

import sahara.db.migration
from sahara.db.sqlalchemy import api as sa
from sahara.db.sqlalchemy import model_base


LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class BaseWalkMigrationTestCase(object):

    ALEMBIC_CONFIG = alembic_config.Config(
        os.path.join(os.path.dirname(sahara.db.migration.__file__),
                     'alembic.ini')
    )

    ALEMBIC_CONFIG.sahara_config = CONF

    def _configure(self, engine):
        """For each type of repository we should do some of configure steps.

        For migrate_repo we should set under version control our database.
        For alembic we should configure database settings. For this goal we
        should use oslo_config and openstack.commom.db.sqlalchemy.session with
        database functionality (reset default settings and session cleanup).
        """
        CONF.set_override('connection', str(engine.url), group='database')
        sa.cleanup()

    def _alembic_command(self, alembic_command, engine, *args, **kwargs):
        """Most of alembic command return data into output.

        We should redefine this setting for getting info.
        """
        self.ALEMBIC_CONFIG.stdout = buf = io.StringIO()
        CONF.set_override('connection', str(engine.url), group='database')
        sa.cleanup()
        getattr(command, alembic_command)(*args, **kwargs)
        res = buf.getvalue().strip()
        LOG.debug('Alembic command {command} returns: {result}'.format(
                  command=alembic_command, result=res))
        sa.cleanup()
        return res

    def _get_versions(self):
        """Stores a list of versions.

        Since alembic version has a random algorithm of generation
        (SA-migrate has an ordered autoincrement naming) we should store
        a list of versions (version for upgrade)
        for successful testing of migrations in up mode.
        """

        env = alembic_script.ScriptDirectory.from_config(self.ALEMBIC_CONFIG)
        versions = []
        for rev in env.walk_revisions():
            versions.append(rev.revision)

        versions.reverse()
        return versions

    def walk_versions(self, engine=None):
        # Determine latest version script from the repo, then
        # upgrade from 1 through to the latest, with no data
        # in the databases. This just checks that the schema itself
        # upgrades successfully.

        self._configure(engine)
        versions = self._get_versions()
        for ver in versions:
            self._migrate_up(engine, ver, with_data=True)

    def _get_version_from_db(self, engine):
        """Returns latest version from db for each type of migrate repo."""

        conn = engine.connect()
        try:
            context = migration.MigrationContext.configure(conn)
            version = context.get_current_revision() or '-1'
        finally:
            conn.close()
        return version

    def _migrate(self, engine, version, cmd):
        """Base method for manipulation with migrate repo.

        It will upgrade or downgrade the actual database.
        """

        self._alembic_command(cmd, engine, self.ALEMBIC_CONFIG, version)

    def _migrate_up(self, engine, version, with_data=False):
        """migrate up to a new version of the db.

        We allow for data insertion and post checks at every
        migration version with special _pre_upgrade_### and
        _check_### functions in the main test.
        """
        # NOTE(sdague): try block is here because it's impossible to debug
        # where a failed data migration happens otherwise
        check_version = version
        try:
            if with_data:
                data = None
                pre_upgrade = getattr(
                    self, "_pre_upgrade_%s" % check_version, None)
                if pre_upgrade:
                    data = pre_upgrade(engine)
            self._migrate(engine, version, 'upgrade')
            self.assertEqual(version, self._get_version_from_db(engine))
            if with_data:
                check = getattr(self, "_check_%s" % check_version, None)
                if check:
                    check(engine, data)
        except Exception:
            LOG.error("Failed to migrate to version {version} on engine "
                      "{engine}".format(version=version, engine=engine))
            raise


class TestModelsMigrationsSync(t_m.ModelsMigrationsSync):
    """Class for comparison of DB migration scripts and models.

    Allows to check if the DB schema obtained by applying of migration
    scripts is equal to the one produced from models definitions.
    """

    ALEMBIC_CONFIG = alembic_config.Config(
        os.path.join(os.path.dirname(sahara.db.migration.__file__),
                     'alembic.ini')
    )
    ALEMBIC_CONFIG.sahara_config = CONF

    def get_engine(self):
        return self.engine

    def db_sync(self, engine):
        CONF.set_override('connection', str(engine.url), group='database')
        alembic.command.upgrade(self.ALEMBIC_CONFIG, 'head')

    def get_metadata(self):
        return model_base.SaharaBase.metadata



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from sahara.plugins import exceptions as ex
from sahara.service.edp import hdfs_helper as helper
from sahara.tests.unit import base


class HDFSHelperTestCase(base.SaharaTestCase):
    def setUp(self):
        super(HDFSHelperTestCase, self).setUp()

        self.cluster = mock.MagicMock()
        self.cluster.id = '1axx'

    def test_create_hbase_common_lib_no_ex(self):

        def _command(a):
            if a == 'hbase classpath':
                return [0, 'april:may.jar:june']

        self.cluster.execute_command.side_effect = _command

        helper.create_hbase_common_lib(self.cluster)
        calls = [
            mock.call(('sudo su - -c "hdfs dfs -mkdir -p '
                       '/user/sahara-hbase-lib" hdfs')),
            mock.call('hbase classpath'),
            mock.call(('sudo su - -c "hdfs dfs -put -p may.jar '
                      '/user/sahara-hbase-lib" hdfs'))]
        self.cluster.execute_command.assert_has_calls(calls)

    def test_create_hbase_common_lib_ex(self):

        def _command(a):
            if a == 'hbase classpath':
                return [1, 'april:may.jar:june']

        self.cluster.execute_command.side_effect = _command

        self.assertRaises(ex.RequiredServiceMissingException,
                          helper.create_hbase_common_lib,
                          self.cluster)

    def test_copy_from_local(self):
        helper.copy_from_local(self.cluster, 'Galaxy', 'Earth', 'BigBang')
        self.cluster.execute_command.assert_called_once_with(
            'sudo su - -c "hdfs dfs -copyFromLocal Galaxy Earth" BigBang')

    def test_move_from_local(self):
        helper.move_from_local(self.cluster, 'Galaxy', 'Earth', 'BigBang')
        self.cluster.execute_command.assert_called_once_with(
            'sudo su - -c "hdfs dfs -copyFromLocal Galaxy Earth" BigBang '
            '&& sudo rm -f Galaxy')

    def test_create_dir_hadoop1(self):
        helper.create_dir_hadoop1(self.cluster, 'Earth', 'BigBang')
        self.cluster.execute_command.assert_called_once_with(
            'sudo su - -c "hdfs dfs -mkdir Earth" BigBang')

    def test_create_dir_hadoop2(self):
        helper.create_dir_hadoop2(self.cluster, 'Earth', 'BigBang')
        self.cluster.execute_command.assert_called_once_with(
            'sudo su - -c "hdfs dfs -mkdir -p Earth" BigBang')

    @mock.patch('sahara.utils.cluster.generate_etc_hosts')
    @mock.patch('sahara.plugins.utils.get_instances')
    @mock.patch('sahara.conductor.api.LocalApi.cluster_get_all')
    def test_get_cluster_hosts_information_smthg_wrong(self, mock_get_all,
                                                       mock_get_inst,
                                                       mock_generate):
        res = helper._get_cluster_hosts_information('host', self.cluster)
        self.assertIsNone(res)

    @mock.patch('sahara.context.ctx')
    @mock.patch('sahara.utils.cluster.generate_etc_hosts')
    @mock.patch('sahara.plugins.utils.get_instances')
    @mock.patch('sahara.conductor.api.LocalApi.cluster_get_all')
    def test_get_cluster_hosts_information_c_id(self, mock_get_all,
                                                mock_get_inst, mock_generate,
                                                mock_ctx):
        cluster = mock.MagicMock()
        cluster.id = '1axx'
        instance = mock.MagicMock()
        instance.instance_name = 'host'
        mock_get_all.return_value = [cluster]
        mock_get_inst.return_value = [instance]
        res = helper._get_cluster_hosts_information('host', self.cluster)
        self.assertIsNone(res)

    @mock.patch('sahara.context.ctx')
    @mock.patch('sahara.utils.cluster.generate_etc_hosts')
    @mock.patch('sahara.plugins.utils.get_instances')
    @mock.patch('sahara.conductor.api.LocalApi.cluster_get_all')
    def test_get_cluster_hosts_information_i_name(self, mock_get_all,
                                                  mock_get_inst, mock_generate,
                                                  mock_ctx):
        cluster = mock.MagicMock()
        cluster.id = '1axz'
        instance = mock.MagicMock()
        instance.instance_name = 'host'
        mock_get_all.return_value = [cluster]
        mock_get_inst.return_value = [instance]
        res = helper._get_cluster_hosts_information('host', self.cluster)
        self.assertEqual(res, mock_generate())

    @mock.patch('sahara.service.edp.hdfs_helper._is_cluster_configured')
    @mock.patch('six.text_type')
    @mock.patch('sahara.plugins.utils.get_instances')
    @mock.patch(('sahara.service.edp.hdfs_helper._get_cluster_hosts_'
                 'information'))
    def test_configure_cluster_for_hdfs(self, mock_helper, mock_get, mock_six,
                                        cluster_conf):
        cluster_conf.return_value = False
        inst = mock.MagicMock()
        inst.remote = mock.MagicMock()
        mock_six.return_value = 111
        str1 = '/tmp/etc-hosts-update.111'
        str2 = ('cat /tmp/etc-hosts-update.111 /etc/hosts | sort | uniq > '
                '/tmp/etc-hosts.111 && cat /tmp/etc-hosts.111 > '
                '/etc/hosts && rm -f /tmp/etc-hosts.111 '
                '/tmp/etc-hosts-update.111')
        mock_get.return_value = [inst]
        helper.configure_cluster_for_hdfs(self.cluster, "www.host.ru")
        inst.remote.assert_has_calls(
            [mock.call(), mock.call().__enter__(),
             mock.call().__enter__().write_file_to(str1, mock_helper()),
             mock.call().__enter__().execute_command(str2, run_as_root=True),
             mock.call().__exit__(None, None, None)])

    @mock.patch('sahara.plugins.utils.get_instances')
    def test_is_cluster_configured(self, mock_get):
        inst = mock.Mock()
        r = mock.MagicMock()
        inst.remote = mock.Mock(return_value=r)
        enter_r = mock.Mock()
        enter_r.execute_command = mock.Mock()
        enter_r.execute_command.return_value = 0, "127.0.0.1 localhost\n" + \
            "127.0.0.2 t1 t1"
        r.__enter__.return_value = enter_r

        cmd = 'cat /etc/hosts'
        host_info = ['127.0.0.1 localhost', '127.0.0.2 t1 t1']
        mock_get.return_value = [inst]
        res = helper._is_cluster_configured(self.cluster, host_info)
        self.assertTrue(res)
        enter_r.execute_command.assert_called_with(cmd)

        enter_r.execute_command.return_value = 0, "127.0.0.1 localhost\n"
        res = helper._is_cluster_configured(self.cluster, host_info)
        self.assertFalse(res)
        enter_r.execute_command.assert_called_with(cmd)

    @mock.patch('six.text_type')
    @mock.patch('os.open')
    def test_put_file_to_hdfs(self, open_get, mock_six):
        open_get.return_value = '/tmp/workflow.xml'
        mock_six.return_value = 111
        helper.put_file_to_hdfs(self.cluster, open_get, 'workflow',
                                '/tmp', 'hdfs')
        self.cluster.execute_command.assert_called_once_with(
            'sudo su - -c "hdfs dfs -copyFromLocal /tmp/workflow.111'
            ' /tmp/workflow" hdfs && sudo rm -f /tmp/workflow.111')



# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import sys

from oslo_config import cfg
from oslo_log import log
import pkg_resources as pkg

from sahara.db.templates import api
from sahara import version

LOG = log.getLogger(__name__)

CONF = cfg.CONF


def extra_option_checks():

    if not CONF.database.connection:
        print("No database connection string was specified in configuration",
              file=sys.stderr)
        sys.exit(1)

    if CONF.command.name in ['update', 'delete']:
        if CONF.command.plugin_version and not CONF.command.plugin_name:
            print("The --plugin-version option is not valid "
                  "without --plugin-name", file=sys.stderr)
            sys.exit(-1)

    if CONF.command.name == "update":
        # Options handling probably needs some refactoring in the future.
        # For now, though, we touch the conductor which ultimately touches
        # the plugins.base. Use the "plugins" option there as a default
        # list of plugins to process, since those are the plugins that
        # will be loaded by Sahara
        if not CONF.command.plugin_name:
            if "plugins" in CONF and CONF.plugins:
                LOG.info("Using plugin list {plugins} from "
                         "config".format(plugins=CONF.plugins))
            else:
                print("No plugins specified with --plugin-name "
                      "or config", file=sys.stderr)
                sys.exit(-1)


def add_command_parsers(subparsers):
    # Note, there is no 'list' command here because the client
    # or REST can be used for list operations. Default templates
    # will display, and templates will show the 'is_default' field.

    def add_id(parser):
        parser.add_argument('--id', required=True,
                            help='The id of the default '
                                 'template to delete')

    def add_tenant_id(parser):
        parser.add_argument('-t', '--tenant_id', required=True,
                            help='Tenant ID for database operations.')

    def add_name_and_tenant_id(parser):
        parser.add_argument('--name', dest='template_name', required=True,
                            help='Name of the default template')
        add_tenant_id(parser)

    def add_plugin_name_and_version(parser, require_plugin_name=False):

        plugin_name_help = ('Only process templates containing '
                            'a "plugin_name" field matching '
                            'one of these values.')

        if not require_plugin_name:
            extra = (' The default list of plugin names '
                     'is taken from the "plugins" parameter in '
                     'the [DEFAULT] config section.')
            plugin_name_help += extra

        parser.add_argument('-p', '--plugin-name', nargs="*",
                            required=require_plugin_name,
                            help=plugin_name_help)

        parser.add_argument('-pv', '--plugin-version', nargs="*",
                            help='Only process templates containing a '
                                 '"hadoop_version" field matching one of '
                                 'these values. This option is '
                                 'only valid if --plugin-name is specified '
                                 'as well. A version specified '
                                 'here may optionally be prefixed with a '
                                 'plugin name and a dot, for example '
                                 '"vanilla.1.2.1". Dotted versions only '
                                 'apply to the plugin named in the '
                                 'prefix. Versions without a prefix apply to '
                                 'all plugins.')

    fname = pkg.resource_filename(version.version_info.package,
                                  "plugins/default_templates")
    # update command
    parser = subparsers.add_parser('update',
                                   help='Update the default template set')
    parser.add_argument('-d', '--directory',
                        default=fname,
                        help='Template directory. Default is %s' % fname)
    parser.add_argument('-n', '--norecurse', action='store_true',
                        help='Do not descend into subdirectories')

    add_plugin_name_and_version(parser)
    add_tenant_id(parser)
    parser.set_defaults(func=api.do_update)

    # delete command
    parser = subparsers.add_parser('delete',
                                   help='Delete default templates '
                                        'by plugin and version')
    add_plugin_name_and_version(parser, require_plugin_name=True)
    add_tenant_id(parser)
    parser.set_defaults(func=api.do_delete)

    # node-group-template-delete command
    parser = subparsers.add_parser('node-group-template-delete',
                                   help='Delete a default '
                                        'node group template by name')
    add_name_and_tenant_id(parser)
    parser.set_defaults(func=api.do_node_group_template_delete)

    # cluster-template-delete command
    parser = subparsers.add_parser('cluster-template-delete',
                                   help='Delete a default '
                                        'cluster template by name')
    add_name_and_tenant_id(parser)
    parser.set_defaults(func=api.do_cluster_template_delete)

    # node-group-template-delete-id command
    parser = subparsers.add_parser('node-group-template-delete-id',
                                   help='Delete a default '
                                        'node group template by id')
    add_id(parser)
    parser.set_defaults(func=api.do_node_group_template_delete_by_id)

    # cluster-template-delete-id command
    parser = subparsers.add_parser('cluster-template-delete-id',
                                   help='Delete a default '
                                        'cluster template by id')
    add_id(parser)
    parser.set_defaults(func=api.do_cluster_template_delete_by_id)


command_opt = cfg.SubCommandOpt('command',
                                title='Command',
                                help='Available commands',
                                handler=add_command_parsers)
CONF.register_cli_opt(command_opt)


def unregister_extra_cli_opt(name):
    try:
        for cli in CONF._cli_opts:
            if cli['opt'].name == name:
                CONF.unregister_opt(cli['opt'])
    except Exception:
        pass


# Remove a few extra CLI opts that we picked up via imports
# Do this early so that they do not appear in the help
for extra_opt in ["log-exchange", "host", "port"]:
    unregister_extra_cli_opt(extra_opt)


def main():
    # TODO(tmckay): Work on restricting the options
    # pulled in by imports which show up in the help.
    # If we find a nice way to do this the calls to
    # unregister_extra_cli_opt() can be removed
    CONF(project='sahara')

    # For some reason, this is necessary to clear cached values
    # and re-read configs.  For instance, if this is not done
    # here the 'plugins' value will not reflect the value from
    # the config file on the command line
    CONF.reload_config_files()
    log.setup(CONF, "sahara")

    # If we have to enforce extra option checks, like one option
    # requires another, do it here
    extra_option_checks()

    # Since this may be scripted, record the command in the log
    # so a user can know exactly what was done
    LOG.info("Command: {command}".format(command=' '.join(sys.argv)))

    api.set_logger(LOG)
    api.set_conf(CONF)

    CONF.command.func()

    LOG.info("Finished {command}".format(command=CONF.command.name))
